#!/usr/bin/python

import os, glob, optparse, shutil, warnings, pickle, math, copy, pickle, matplotlib
import matplotlib.pyplot as plt
import numpy as np
import scipy.signal, scipy.stats
from scipy import optimize
import seismon.NLNM, seismon.html
import seismon.eqmon, seismon.utils
from matplotlib import cm

try:
    import gwpy.time, gwpy.timeseries
    import gwpy.frequencyseries, gwpy.spectrogram
except:
    print("gwpy import fails... no plotting possible.")

__author__ = "Michael Coughlin <michael.coughlin@ligo.org>"
__date__ = "2012/8/26"
__version__ = "0.1"

# =============================================================================
#
#                               DEFINITIONS
#
# =============================================================================

def save_data(params,channel,gpsStart,gpsEnd,data,attributeDics):
    """@saves spectral data in text files.

    @param params
        seismon params dictionary
    @param channel
        seismon channel structure
    @param gpsStart
        start gps
    @param gpsStart
        start gps
    @param gpsEnd
        end gps
    @param data
        spectral data structure 
    """

    ifo = seismon.utils.getIfo(params)

    psdDirectory = params["dirPath"] + "/Text_Files/PSD/" + channel.station_underscore + "/" + str(params["fftDuration"])
    seismon.utils.mkdir(psdDirectory)

    powerlawDirectory = params["dirPath"] + "/Text_Files/Powerlaw/" + channel.station_underscore + "/" + str(params["fftDuration"])
    seismon.utils.mkdir(powerlawDirectory)

    EQpowerlawDirectory = params["dirPath"] + "/Text_Files/EQPowerlaw/" + channel.station_underscore + "/" + str(params["fftDuration"])
    seismon.utils.mkdir(EQpowerlawDirectory)

    fftDirectory = params["dirPath"] + "/Text_Files/FFT/" + channel.station_underscore + "/" + str(params["fftDuration"])
    seismon.utils.mkdir(fftDirectory)

    timeseriesDirectory = params["dirPath"] + "/Text_Files/Timeseries/" + channel.station_underscore + "/" + str(params["fftDuration"])
    seismon.utils.mkdir(timeseriesDirectory)

    accelerationDirectory = params["dirPath"] + "/Text_Files/Acceleration/" + channel.station_underscore + "/" + str(params["fftDuration"])
    seismon.utils.mkdir(accelerationDirectory)

    displacementDirectory = params["dirPath"] + "/Text_Files/Displacement/" + channel.station_underscore + "/" + str(params["fftDuration"])
    seismon.utils.mkdir(displacementDirectory)

    earthquakesDirectory = params["dirPath"] + "/Text_Files/Earthquakes/" + channel.station_underscore + "/" + str(params["fftDuration"])
    seismon.utils.mkdir(earthquakesDirectory)

    spectrogramDirectory = params["dirPath"] + "/Text_Files/Spectrogram/" + channel.station_underscore + "/" + str(params["fftDuration"])
    seismon.utils.mkdir(spectrogramDirectory)

    freq = np.array(data["dataASD"].frequencies)

    psdFile = os.path.join(psdDirectory,"%d-%d.txt"%(gpsStart,gpsEnd))
    f = open(psdFile,"wb")
    for i in range(len(freq)):
        f.write("%e %e\n"%(freq[i],data["dataASD"][i].value))
    f.close()

    freq = np.array(data["dataFFT"].frequencies)

    fftFile = os.path.join(fftDirectory,"%d-%d.txt"%(gpsStart,gpsEnd))
    f = open(fftFile,"wb")
    for i in range(len(freq)):
        f.write("%e %e %e\n"%(freq[i],data["dataFFT"][i].value.real,data["dataFFT"][i].value.imag))
    f.close()

    tt = np.array(data["dataLowpass"].times)
    timeseriesFile = os.path.join(timeseriesDirectory,"%d-%d.txt"%(gpsStart,gpsEnd))
    f = open(timeseriesFile,"wb")
    f.write("%.10f %e\n"%(tt[np.argmin(data["dataLowpass"].value)],np.min(data["dataLowpass"].value)))
    f.write("%.10f %e\n"%(tt[np.argmax(data["dataLowpass"].value)],np.max(data["dataLowpass"].value)))
    f.close()

    ttacc = np.array(data["dataLowpassAcc"].times)
    accelerationFile = os.path.join(accelerationDirectory,"%d-%d.txt"%(gpsStart,gpsEnd))
    f = open(accelerationFile,"wb")
    f.write("%.10f %e\n"%(ttacc[np.argmin(data["dataLowpassAcc"].value)],np.min(data["dataLowpassAcc"].value)))
    f.write("%.10f %e\n"%(ttacc[np.argmax(data["dataLowpassAcc"].value)],np.max(data["dataLowpassAcc"].value)))
    f.close()

    ttdisp = np.array(data["dataLowpassDisp"].times)
    displacementFile = os.path.join(displacementDirectory,"%d-%d.txt"%(gpsStart,gpsEnd))
    f = open(displacementFile,"wb")    
    f.write("%.10f %e\n"%(ttdisp[np.argmin(data["dataLowpassDisp"].value)],np.min(data["dataLowpassDisp"].value)))
    f.write("%.10f %e\n"%(ttdisp[np.argmax(data["dataLowpassDisp"].value)],np.max(data["dataLowpassDisp"].value)))
    f.close()

    specgram = data["dataSpecgram"]
    freq = np.array(specgram.frequencies)
    times = np.array(specgram.times)

    spectrogramFile = os.path.join(spectrogramDirectory,"%d-%d.txt"%(gpsStart,gpsEnd))
    f = open(spectrogramFile,"wb")
    f.write("-1")
    for jj in range(len(times)):
        f.write(" %.10f "%times[jj])
    f.write("\n")
    for ii in range(len(freq)):
        f.write("%.10f"%freq[ii])
        for jj in range(len(times)):
                f.write(" %e "%(specgram[jj,ii].value))
        f.write("\n")
    f.close()

    if params["doPowerLawFit"]:
        xdata = freq
        ydata = np.array(data["dataASD"])

        indexes = np.where((xdata >= 0.05) & (xdata<=1))[0]
        xdata = xdata[indexes]
        ydata = ydata[indexes]

        logx = np.log10(xdata)
        logy = np.log10(ydata)

        fitfunc = lambda p, x: p[0] + p[1] * x
        errfunc = lambda p, x, y: (y - fitfunc(p, x))

        out,success = optimize.leastsq(errfunc, [1,-1],args=(logx,logy),maxfev=3000)

        index = out[1]
        amp = 10.0**out[0]

        powerlawFile = os.path.join(powerlawDirectory,"%d-%d.txt"%(gpsStart,gpsEnd))
        f = open(powerlawFile,"wb")
        f.write("%e %e\n"%(index,amp))
        f.close()


    if params["doEarthquakesTrips"]:

        tripsDirectory = params["dirPath"] + "/Text_Files/Trips/" + channel.station_underscore + "/" + str(params["fftDuration"])
        seismon.utils.mkdir(tripsDirectory)

        trips = []
        platforms = []
        stages = []
        trip_lines = [line.strip() for line in open(params["tripsTextFile"])]
        for line in trip_lines:
            lineSplit = line.split(" ")
            trips.append(float(lineSplit[0]))
            platforms.append(lineSplit[1])
            stages.append(lineSplit[2])

        count = 1
        for trip,platform,stage in zip(trips,platforms,stages):
            if (trip >= gpsStart) and (trip <= gpsEnd):
                    tt_diff = np.absolute(tt - trip)
                    index = np.argmin(tt_diff)

                    tripsDirectory = params["dirPath"] + "/Text_Files/Trips/" + channel.station_underscore + "/" + str(params["fftDuration"]) + "/" + platform + "/" + stage
                    seismon.utils.mkdir(tripsDirectory)

                    tripsFile = os.path.join(tripsDirectory,"%d.txt"%(trip))
                    f = open(tripsFile,"wb")
                    f.write("%d %e\n"%(tt[index],data["dataLowpass"].value[index]))
                    f.close()

    for attributeDic in attributeDics:

        if not "Arbitrary" in attributeDic["traveltimes"]:
            continue  

        if params["ifo"] == "IRIS":
            attributeDic = seismon.eqmon.ifotraveltimes_loc(attributeDic, "IRIS", channel.latitude, channel.longitude)
            traveltimes = attributeDic["traveltimes"]["IRIS"]
        else:
            traveltimes = attributeDic["traveltimes"][ifo]

        Ptime = max(traveltimes["Ptimes"])
        Stime = max(traveltimes["Stimes"])
        Rtwotime = max(traveltimes["Rtwotimes"])
        RthreePointFivetime = max(traveltimes["RthreePointFivetimes"])
        Rfivetime = max(traveltimes["Rfivetimes"])
        distance = max(traveltimes["Distances"])

        indexes = np.intersect1d(np.where(tt >= Rfivetime)[0],np.where(tt <= Rtwotime)[0])

        if len(indexes) == 0:
            continue

        indexMin = np.min(indexes)
        indexMax = np.max(indexes)
        ttCut = tt[indexes]
        dataCut = data["dataLowpass"][indexMin:indexMax]

        ampMax = np.max(dataCut.value)
        ttMax = ttCut[np.argmax(dataCut.value)]
        ttDiff = ttMax - attributeDic["GPS"] 
        velocity = distance / ttDiff
        velocity = velocity / 1000.0
 
        earthquakesFile = os.path.join(earthquakesDirectory,"%s.txt"%(attributeDic["eventName"]))
        f = open(earthquakesFile,"wb")
        f.write("%.10f %e %e %e %e %.1f %.1f %.1f\n"%(ttMax,ttDiff,distance,velocity,ampMax,Ptime,Stime,RthreePointFivetime))
        f.close()

        if params["doPowerLawFit"]:
            xdata = freq
            ydata = np.array(data["dataASD"])

            indexes = np.where((xdata >= 0.05) & (xdata<=1))[0]
            xdata = xdata[indexes]
            ydata = ydata[indexes]

            logx = np.log10(xdata)
            logy = np.log10(ydata)

            fitfunc = lambda p, x: p[0] + p[1] * x
            errfunc = lambda p, x, y: (y - fitfunc(p, x))

            out,success = optimize.leastsq(errfunc, [1,-1],args=(logx,logy),maxfev=3000)

            index = out[1]
            amp = 10.0**out[0]

            powerlawFile = os.path.join(EQpowerlawDirectory,"%s.txt"%(attributeDic["eventName"]))
            f = open(powerlawFile,"wb")
            f.write("%e %e\n"%(index,amp))
            f.close()

def calculate_spectra(params,channel,dataFull):
    """@calculate spectral data

    @param params
        seismon params dictionary
    @param channel
        seismon channel structure
    @param dataFull
        timeseries data structure
    """

    fs = channel.samplef # 1 ns -> 1 GHz

    if channel.station == "H1:ISI-GND_BRS_ETMX_RY_OUT_DQ":
        cutoff_high = 0.05 # 10 MHz
        cutoff_low = 0.3
    else:
        cutoff_high = 0.5 # 10 MHz
        cutoff_low = 0.1
        cutoff_band = 0.01
    n = 3
    worN = 16384
    B_low, A_low = scipy.signal.butter(n, cutoff_low / (fs / 2.0), btype='lowpass')
    #B_low, A_low = scipy.signal.butter(n, [cutoff_band/(fs / 2.0), cutoff_low/(fs / 2.0)], btype='band')
    #w_low, h_low = scipy.signal.freqz(B_low,A_low)
    w_low, h_low = scipy.signal.freqz(B_low,A_low,worN=worN)
    B_high, A_high = scipy.signal.butter(n, cutoff_high / (fs / 2.0), btype='highpass') 
    w_high, h_high = scipy.signal.freqz(B_high,A_high,worN=worN)

    w = w_high * (fs / (2.0*np.pi))

    B_band, A_band = scipy.signal.butter(n, cutoff_band / (fs / 2.0), btype='high')

    if params["doPlots"]:

        plotDirectory = params["path"] + "/" + channel.station_underscore
        seismon.utils.mkdir(plotDirectory)

        pngFile = os.path.join(plotDirectory,"bode.png")
        kwargs = {'logx':True}
        plot = gwpy.plotter.BodePlot(figsize=[14,8],**kwargs)
        plot.add_filter((B_low,A_low),frequencies=w_high,label="lowpass")
        plot.add_filter((B_high,A_high),frequencies=w_high,label="highpass")
        plot.add_legend(loc=1,prop={'size':10})
        plot.save(pngFile,dpi=200)
        plot.close()

    #dataLowpass = dataFull.copy()
    #lowpassdata = scipy.signal.filtfilt(B_low, A_low, np.array(dataFull.value))
    #lowpassdata[:2*channel.samplef] = lowpassdata[2*channel.samplef]
    #lowpassdata[-2*channel.samplef:] = lowpassdata[-2*channel.samplef]
    #lowpassdata[np.isnan(lowpassdata)] = 0.0
    dataLowpass = dataFull.lowpass(cutoff_low)

    accdata = np.diff(dataLowpass.value)/dataLowpass.dx.value
    dataLowpassAcc = gwpy.timeseries.TimeSeries(accdata, unit=dataFull.unit, sample_rate = 1/dataFull.dx.value, epoch = dataFull.epoch, dtype=float)

    dispdata = scipy.integrate.cumtrapz(dataLowpass.value, dx=dataLowpass.dx.value)
    dataLowpassDisp = gwpy.timeseries.TimeSeries(dispdata, unit=dataFull.unit, sample_rate = 1/dataFull.dx.value, epoch = dataFull.epoch, dtype=float)

    dataHighpass = dataFull.highpass(1.0)

    # calculate spectrum
    NFFT = params["fftDuration"]
    #window = None
    #dataASD = dataFull.asd()
    dataASD = dataFull.asd(fftlength=NFFT,overlap=None,method='welch')
    freq = np.array(dataASD.frequencies)
    indexes = np.where((freq >= params["fmin"]) & (freq <= params["fmax"]))[0]
    dataASD = np.array(dataASD.value)
    freq = freq[indexes]
    dataASD = dataASD[indexes]
    dataASD = gwpy.frequencyseries.FrequencySeries(dataASD, f0=np.min(freq), df=(freq[1]-freq[0]))

    nfft = len(dataFull.value)
    dataFFT = dataFull.fft(nfft=nfft)
    freqFFT = np.array(dataFFT.frequencies)
    dataFFT = np.array(dataFFT)
    dataFFTreal = np.interp(freq,freqFFT,dataFFT.real)
    dataFFTimag = np.interp(freq,freqFFT,dataFFT.imag)
    dataFFT = dataFFTreal + 1j*dataFFTimag
    dataFFT = gwpy.frequencyseries.FrequencySeries(dataFFT, f0=np.min(freqFFT), df=(freqFFT[1]-freqFFT[0]))

    # manually set units (units in CIS aren't correct)
    #dataASD.unit = 'counts/Hz^(1/2)'
    #dataFFT.unit = 'counts/Hz^(1/2)'
    dataASD.override_unit('count/Hz^(1/2)')
    dataFFT.override_unit('count/Hz^(1/2)')

    data = {}
    data["dataFull"] = dataFull
    data["dataLowpass"] = dataLowpass
    data["dataLowpassAcc"] = dataLowpassAcc
    data["dataLowpassDisp"] = dataLowpassDisp
    data["dataHighpass"] = dataHighpass
    data["dataASD"] = dataASD
    data["dataFFT"] = dataFFT

    if params["doEarthquakesHilbert"]:

        dataHilbert = scipy.signal.hilbert(dataLowpass).imag
        dataHilbert = dataHilbert.view(dataLowpass.__class__)
        dataHilbert.sample_rate =  dataFull.sample_rate
        dataHilbert.epoch = dataFull.epoch
        data["dataHilbert"] = dataHilbert

    return data

def apply_calibration(params,channel,data):
    """@applies calibration to necessary channels

    @param params
        seismon params dictionary
    @param channel
        seismon channel structure
    @param data
        spectral data structure
    """  

    if False:
    #if ("L4C" in channel.station) or ("GS13" in channel.station):

        zeros = [0,0,0]
        poles = [0.70711 + 0.70711*1j , 0.70711 - 0.70711*1j]
        gain = 1

        #poles = -2*np.pi*poles
        #zeros = -2*np.pi*zeros

        filt = [[(-1)*np.complex(0.70711,0.70711),(-1)*np.complex(0.70711,-0.70711)],[0,0],1]
        b, a = scipy.signal.zpk2tf(*filt)

        #b = [1,0,0,0];
        #a = [0,1,-1.414,1];
        w, h = scipy.signal.freqz(b, a)

        f = data["dataASD"].frequencies.value
      
        # Divide by f to get to displacement
        data["dataASD"]/=f
        # Filter spectrum
        filt = [b,a] 
        data["dataASD"].filter(*filt,inplace=True)
        fresp = abs(scipy.signal.freqs(b, a, f)[1])
        # Multiply by f to get to velocity
        data["dataASD"]*=f

        if params["doPlots"]:

            plotDirectory = params["path"] + "/" + channel.station_underscore
            seismon.utils.mkdir(plotDirectory)

            pngFile = os.path.join(plotDirectory,"calibration.png")
            plot = gwpy.plotter.Plot(figsize=[14,8])
            plot.add_line(f,fresp)
            plot.xlim = [params["fmin"],params["fmax"]]
            plot.xlabel = "Frequency [Hz]"
            plot.ylabel = "Response"
            plot.title = channel.station.replace("_","\_")
            plot.axes[0].set_xscale("log")
            plot.axes[0].set_yscale("log")
            plot.save(pngFile,dpi=200)
            plot.close()

    elif channel.station == "H1:ISI-GND_BRS_ETMX_RY_OUT_DQ":

        f = data["dataASD"].frequencies.value
        # Multiply by f to get to velocity
        data["dataASD"]/=(2*np.pi*f)

    elif "IRIS:II:BFO:00:LA1" == channel.station:

        z = []
        p = [4.56775E-03,-2.06901E-04]
        p = 1
        z = [-4.56775E-03,-2.06901E-04]
        k = 2.51043E-03
        k = 1

        b, a = scipy.signal.zpk2tf(z, p, k)

        f = data["dataASD"].frequencies.value

        # Filter spectrum
        data["dataASD"].filterba(a,b,inplace=True)
        data["dataASD"]*=9.81
        data["dataASD"]/=f
        fresp = abs(scipy.signal.freqs(a, b, f)[1])

        if params["doPlots"]:

            plotDirectory = params["path"] + "/" + channel.station_underscore
            seismon.utils.mkdir(plotDirectory)

            pngFile = os.path.join(plotDirectory,"calibration.png")
            plot = gwpy.plotter.Plot(figsize=[14,8])
            plot.add_line(f,fresp)
            plot.xlim = [params["fmin"],params["fmax"]]
            plot.xlabel = "Frequency [Hz]"
            plot.ylabel = "Response"
            plot.title = channel.station.replace("_","\_")
            plot.axes[0].set_xscale("log")
            plot.axes[0].set_yscale("log")
            plot.save(pngFile,dpi=200)
            plot.close()

    return data

def calculate_picks(params,channel,data):

    if params["doEarthquakesPicks"]:
        import obspy.signal

        nsta = int(2.5 * channel.samplef)
        nlta = int(10.0 * channel.samplef)
        cft = obspy.signal.trigger.recSTALTA(data["dataFull"].value, nsta, nlta)

        thres1 = 0.9 * np.max(cft)
        thres2 = 0.5
        on_off = obspy.signal.triggerOnset(cft, thres1, thres2)
        on_off = np.array(on_off) / channel.samplef
        on_off = data["dataFull"].epoch.val + on_off

    else:
        on_off = []

    data["on_off"] = on_off
   
    return data

def spectra(params, channel, segment):
    """@calculates spectral data for given channel and segment.

    @param params
        seismon params dictionary
    @param channel
        seismon channel structure
    @param segment
        [start,end] gps
    """

    ifo = seismon.utils.getIfo(params)

    gpsStart = segment[0]
    gpsEnd = segment[1]
   
    # set the times
    duration = np.ceil(gpsEnd-gpsStart)

    # make timeseries
    dataFull = seismon.utils.retrieve_timeseries(params, channel, segment)
    print("data read in...")
    if dataFull == []:
        return 

    dataFull = dataFull / channel.calibration
    indexes = np.where(np.isnan(dataFull.value))[0]
    meanSamples = np.median(np.ma.masked_array(dataFull.value,np.isnan(dataFull.value)))
    for index in indexes:
        dataFull[index] = meanSamples
    meanSamples = np.median(np.ma.masked_array(dataFull.value,np.isnan(dataFull.value)))
    dataFull -= meanSamples * dataFull.unit

    if np.mean(dataFull.value) == 0.0:
        print("data only zeroes... continuing\n")
        return
    if len(dataFull.value) < 2*channel.samplef:
        print("timeseries too short for analysis... continuing\n")
        return

    if channel.station == "H1:ISI-GND_BRS_ETMX_RY_OUT_DQ":
        dataFull *= 9.81

    print("calculating spectra...")
    data = calculate_spectra(params,channel,dataFull)
    print("calculating calibration...")
    data = apply_calibration(params,channel,data)
    data = calculate_picks(params,channel,data)

    # calculate spectrogram
    specgram = dataFull.spectrogram(params["fftDuration"])
    specgram **= 1/2.
    data["dataSpecgram"] = specgram

    #speclog = specgram.to_logf()
    #medratio = speclog.ratio('median')
    medratio = specgram.ratio('median')

    if params["doEarthquakes"]:
        earthquakesDirectory = os.path.join(params["path"],"earthquakes")
        earthquakesXMLFile = os.path.join(earthquakesDirectory,"earthquakes.xml")
        attributeDics = seismon.utils.read_eqmons(earthquakesXMLFile)
    else:
        attributeDics = []
    #attributeDics = [attributeDics[0]]

    save_data(params,channel,gpsStart,gpsEnd,data,attributeDics)

    if params["doPlots"]:

        plotDirectory = params["path"] + "/" + channel.station_underscore
        seismon.utils.mkdir(plotDirectory)        

        pngFile = os.path.join(plotDirectory,"timeseries.png")
        plot = gwpy.plotter.TimeSeriesPlot(figsize=[14,8])

        dataHighpass = data["dataHighpass"].resample(16)
        dataFull = data["dataFull"].resample(16)
        dataLowpass = data["dataLowpass"].resample(16)
        dataLowpassAcc = data["dataLowpassAcc"].resample(16)     
        dataLowpassDisp = data["dataLowpassDisp"].resample(16) 

        #dataHighpass = data["dataHighpass"]
        #dataFull = data["dataFull"]
        #dataLowpass = data["dataLowpass"]

        dataHighpass *= 1e6
        #dataFull *= 1e6
        dataLowpass *= 1e6
        dataLowpassAcc *= 1e6
        dataLowpassDisp *= 1e6

        if channel.station == "H1:ISI-GND_BRS_ETMX_RY_OUT_DQ":
            #plot.add_timeseries(dataHighpass,label="data")
            #dataFull *= 1e6*9.81/(2*np.pi*0.1)
            #plot.add_timeseries(dataFull,label="data")

            dataHighpass *= (9.81)/(2*np.pi*0.05)

            plot.add_timeseries(dataHighpass,label="highpass")

            xlim = [plot.xlim[0],plot.xlim[1]]
            ylim = [plot.ylim[0],plot.ylim[1]]

            #ylim = [-0.5*(9.81)/(2*np.pi*0.05),0.5*(9.81)/(2*np.pi*0.05)]

        else:

            plot.add_timeseries(dataHighpass,label="highpass")
            #plot.add_timeseries(dataFull,label="data")
            kwargs = {"linestyle":"-","color":"k"}
            plot.add_timeseries(dataLowpass,label="lowpass",**kwargs)

            xlim = [plot.xlim[0],plot.xlim[1]]
            ylim = [plot.ylim[0],plot.ylim[1]]
        count = 0
        for attributeDic in attributeDics:

            if not "Arbitrary" in attributeDic["traveltimes"]:
                continue

            #if channel.station == "H1:ISI-GND_BRS_ETMX_RY_OUT_DQ":
            #    continue

            if attributeDic["Magnitude"] < params["minMagnitude"]:
                continue

            if params["ifo"] == "IRIS":
                attributeDic = seismon.eqmon.ifotraveltimes_loc(attributeDic, "IRIS", channel.latitude, channel.longitude)
                traveltimes = attributeDic["traveltimes"]["IRIS"]
            else:
                traveltimes = attributeDic["traveltimes"][ifo]

            Ptime = max(traveltimes["Ptimes"])
            Stime = max(traveltimes["Stimes"])
            Rtwotime = max(traveltimes["Rtwotimes"])
            RthreePointFivetime = max(traveltimes["RthreePointFivetimes"])
            Rfivetime = max(traveltimes["Rfivetimes"])

            if params["doEarthquakesVelocityMap"]:
                Rvelocitymaptimes = traveltimes["Rvelocitymaptimes"]
                Rvelocitymaptime = max(traveltimes["Rvelocitymaptimes"])
                Rvelocitymapvelocities = traveltimes["Rvelocitymapvelocities"]

            peak_velocity = traveltimes["Rfamp"][0]
            peak_velocity = peak_velocity * 1e6

            if peak_velocity > ylim[1]:
               ylim[1] = peak_velocity*1.1
            if -peak_velocity < ylim[0]:
               ylim[0] = -peak_velocity*1.1

            if count ==  0:
                kwargs = {"linestyle":"--","color":"r"}
                plot.add_line([Ptime,Ptime],ylim,label="P Est. Arrival",**kwargs)
                plt.text(Ptime+30,ylim[1]-10,'P')
                kwargs = {"linestyle":"--","color":"g"}
                plot.add_line([Stime,Stime],ylim,label="S Est. Arrival",**kwargs)
                plt.text(Stime+30,ylim[1]-10,'S')
                kwargs = {"linestyle":"--","color":"b"}
                plot.add_line([RthreePointFivetime,RthreePointFivetime],ylim,label="3.5 km/s R Est. Arrival",**kwargs)            
                plt.text(RthreePointFivetime+30,ylim[1]-10,'Rf')
                plot.add_line([Rtwotime,Rtwotime],ylim,label="2 km/s R Est. Arrival",**kwargs)
                plot.add_line([Rfivetime,Rfivetime],ylim,label="5 km/s R Est. Arrival",**kwargs)

                if params["doEarthquakesVelocityMap"]:
                    kwargs = {"linestyle":"--","color":"m"}
                    plot.add_line([Rvelocitymaptime,Rvelocitymaptime],ylim,label="Velocity map R Est. Arrival",**kwargs)
                kwargs = {"linestyle":"--","color":"k"}
                plot.add_line(xlim,[peak_velocity,peak_velocity],label="pred. vel.",**kwargs)
                plot.add_line(xlim,[-peak_velocity,-peak_velocity],**kwargs)

                tstart = (Rfivetime + RthreePointFivetime)/2.0
                #tstart = RthreePointFivetime
                tend = Rtwotime
                x = np.arange(tstart,tend,1)
                y = (x - np.min(x))/600.0
                mu = RthreePointFivetime
                lamb = 1 + y[-1]/5.0
                vals = scipy.stats.gamma.pdf(y, lamb)
                vals = np.absolute(vals)
                vals = vals / np.max(vals)
                vals = vals * peak_velocity
                kwargs = {"linestyle":"--","color":"g"}
                #plot.add_line(x,vals,label="Envelope",**kwargs)

            else:
                kwargs = {"linestyle":"--","color":"r"}
                plot.add_line([Ptime,Ptime],ylim,**kwargs)
                kwargs = {"linestyle":"--","color":"g"}
                plot.add_line([Stime,Stime],ylim,**kwargs)
                kwargs = {"linestyle":"--","color":"b"}
                plot.add_line([RthreePointFivetime,RthreePointFivetime],ylim,**kwargs)
                plot.add_line([Rtwotime,Rtwotime],ylim,**kwargs)
                plot.add_line([Rfivetime,Rfivetime],ylim,**kwargs)

                if params["doEarthquakesVelocityMap"]:
                    kwargs = {"linestyle":"--","color":"m"}
                    plot.add_line([Rvelocitymaptime,Rvelocitymaptime],ylim,**kwargs)
                kwargs = {"linestyle":"--","color":"k"}
                plot.add_line(xlim,[peak_velocity,peak_velocity],**kwargs)
                plot.add_line(xlim,[-peak_velocity,-peak_velocity],**kwargs)

            earthquakesDirectory = params["dirPath"] + "/Text_Files/Earthquakes/" + channel.station_underscore + "/" + str(params["fftDuration"])
            earthquakesFile = os.path.join(earthquakesDirectory,"%s.txt"%(attributeDic["eventName"]))
 
            if not os.path.isfile(earthquakesFile):
                continue

            data_out = np.loadtxt(earthquakesFile)
            ttMax = data_out[0]
            kwargs = {"linestyle":"-","color":"k"}
            #plot.add_line([ttMax,ttMax],ylim,label="Max amplitude",**kwargs)

            count = count + 1

        count = 0
        for on_off in data["on_off"]:

            ontime = on_off[0]
            offtime = on_off[1]

            if count ==  0:
                kwargs = {"linestyle":"-.","color":"r"}
                plot.add_line([ontime,ontime],ylim,label="On trigger",**kwargs)
                kwargs = {"linestyle":"-.","color":"b"}
                plot.add_line([offtime,offtime],ylim,label="Off trigger",**kwargs)
            else:
                kwargs = {"linestyle":"-.","color":"r"}
                plot.add_line([ontime,ontime],ylim,**kwargs)
                kwargs = {"linestyle":"-.","color":"b"}
                plot.add_line([offtime,offtime],ylim,**kwargs)
            count = count + 1
 
        if params["doEarthquakesTrips"]:    
            trips = []
            platforms = []
            stages = [] 
            trip_lines = [line.strip() for line in open(params["tripsTextFile"])]
            for line in trip_lines:
                lineSplit = line.split(" ")
                trips.append(float(lineSplit[0]))
                platforms.append(lineSplit[1])
                stages.append(lineSplit[2])

            ylim_diff = ylim[1] - ylim[0]
            perc_diff = ylim_diff * 0.025

            count = 1
            for trip,platform,stage in zip(trips,platforms,stages):
                if (trip >= xlim[0]) and (trip <= xlim[1]):
                    if count ==  1:
                        kwargs = {"linestyle":"-.","color":"m","linewidth":4}
                        plot.add_line([trip,trip],ylim,label="Trip time",**kwargs)
                    else:
                        kwargs = {"linestyle":"-.","color":"m","linewidth":4}
                        plot.add_line([trip,trip],ylim,**kwargs)
                    plt.text(trip+15,ylim[1]-count*perc_diff,'%s %s'%(platform,stage),fontsize=12)
                    count = count + 1

        if channel.station == "H1:ISI-GND_BRS_ETMX_RY_OUT_DQ":
            plot.ylabel = r"Angle [$\mu$rad]"
        elif channel.station == "H1:ISI-GND_BRS_ETMX_DAMPCTRLMON":
            plot.ylabel = r"Counts"
        else:
            plot.ylabel = r"Velocity [$\mu$m/s]"
        plot.title = channel.station.replace("_","\_")
        plot.xlim = xlim
        plot.ylim = ylim
        plot.add_legend(loc=1,prop={'size':10})

        #plot.ylim = [-0.05,0.05]
        plot.save(pngFile)
        #plot.grid = 0
        #pdfFile = os.path.join(plotDirectory,"timeseries.pdf")
        #plot.save(pdfFile)

        try:
            plot.save(pngFile)

            epsFile = os.path.join(plotDirectory,"timeseries.eps")
            plot.save(epsFile)

            pdfFile = os.path.join(plotDirectory,"timeseries.pdf")
            plot.save(pdfFile)

            pngFile = os.path.join(plotDirectory,"timeseries_30.png")
            plot.ylim = [-30,30]
            plot.save(pngFile)

        except:
            pass
        plot.close()

        if params["doEarthquakesHilbert"]:
            dataHilbert = data["dataHilbert"].resample(16)
            dataHilbert *= 1e6

            pngFile = os.path.join(plotDirectory,"hilbert.png")
            plot = gwpy.plotter.TimeSeriesPlot(figsize=[14,8])

            plot.add_timeseries(dataHilbert,label="highpass")

            plot.ylabel = r"Velocity [$\mu$m/s]"
            plot.title = channel.station.replace("_","\_")
            plot.xlim = xlim
            plot.ylim = ylim

            plot.save(pngFile)
            plot.close()

        pngFile = os.path.join(plotDirectory,"acc.png")
        plot = gwpy.plotter.TimeSeriesPlot(figsize=[14,8])

        plot.add_timeseries(dataLowpassAcc,label="highpass")

        plot.ylabel = r"Acceleration [$\mu$m/s$^2$]"
        plot.title = channel.station.replace("_","\_")
        #plot.xlim = xlim
        #plot.ylim = ylim

        plot.save(pngFile)
        plot.close()

        pngFile = os.path.join(plotDirectory,"disp.png")
        plot = gwpy.plotter.TimeSeriesPlot(figsize=[14,8])

        plot.add_timeseries(dataLowpassDisp,label="highpass")

        plot.ylabel = r"Displacement [$\mu$m]"
        plot.title = channel.station.replace("_","\_")
        #plot.xlim = xlim
        #plot.ylim = ylim

        plot.save(pngFile)
        plot.close()

        fl, low, fh, high = seismon.NLNM.NLNM(2)


        pngFile = os.path.join(plotDirectory,"psd.png")
        label = channel.station.replace("_","\_")

        plot = gwpy.plotter.Plot(figsize=[14,8])
        plot.add_frequencyseries(data["dataASD"],label=label)
        kwargs = {"linestyle":"-.","color":"k"}
        plot.add_line(fl, low, label="HNM/LNM", **kwargs)
        plot.add_line(fh, high, **kwargs)
        plot.xlim = [params["fmin"],params["fmax"]]
        plot.ylim = [10**-10, 10**-4]
        #plot.ylim = [10**-8, 10**-2]
        plot.xlabel = "Frequency [Hz]"

        if channel.station == "H1:ISI-GND_BRS_ETMX_RY_OUT_DQ":
            plot.ylabel = r"Angle [$\mu$rad/rtHz]"
        else:
            plot.ylabel = "Amplitude Spectrum [(m/s)/rtHz]"
        plot.title = channel.station.replace("_","\_")
        plot.axes[0].set_xscale("log")
        plot.axes[0].set_yscale("log")
        plot.add_legend(loc=1,prop={'size':10})
        plot.save(pngFile,dpi=200)
        plot.close()

        pngFile = os.path.join(plotDirectory,"psd_linear.png")
        label = channel.station.replace("_","\_")

        if not np.sum(data["dataASD"]) == 0:
            plot = gwpy.plotter.Plot(figsize=[14,8])
            plot.add_spectrum(data["dataASD"],label=label)
            kwargs = {"linestyle":"-.","color":"k"}
            #plot.add_line(fl, low, label="HNM/LNM", **kwargs)
            #plot.add_line(fh, high, **kwargs)
            plot.xlim = [params["fmin"],params["fmax"]]
            #plot.ylim = [10**-10, 10**-4]
            plot.xlabel = "Frequency [Hz]"
            plot.ylabel = "Amplitude Spectrum [(m/s)/rtHz]"
            plot.title = channel.station.replace("_","\_")
            #plot.axes[0].set_xscale("log")
            plot.axes[0].set_yscale("log")
            plot.add_legend(loc=1,prop={'size':10})
            plot.save(pngFile,dpi=200)
            plot.close()
  
        if not len(medratio) == 0:
           pngFile = os.path.join(plotDirectory,"tf.png")
           plot = medratio.plot(figsize=[14,8])
           plot.add_colorbar(log=True, clim=[0.1, 10], label='ASD ratio to median average')
           plot.ylabel = "Frequency [Hz]"
           plot.ylim = [params["fmin"],params["fmax"]]
           plot.axes[0].set_yscale("log")
           plot.xlim = [gpsStart,gpsEnd]
           plot.axes[0].auto_gps_scale()
           plot.show()
           plot.save(pngFile,dpi=200)
           plot.close()                                       

        if params["doEarthquakesVelocityMap"]:
            pngFile = os.path.join(plotDirectory,"velocitymaps.png")
            plot = gwpy.plotter.TimeSeriesPlot(figsize=[14,8])

            for attributeDic in attributeDics:
       
                Rvelocitymaptimes = traveltimes["Rvelocitymaptimes"]
                Rvelocitymaptime = max(traveltimes["Rvelocitymaptimes"])
                Rvelocitymapvelocities = traveltimes["Rvelocitymapvelocities"]

                kwargs = {"linestyle":"-","color":"k"}
                plot.add_line(Rvelocitymaptimes,Rvelocitymapvelocities,label="Velocity Map",**kwargs)

            xlim = [plot.xlim[0],plot.xlim[1]]
            ylim = [plot.ylim[0],plot.ylim[1]]

            kwargs = {"linestyle":"-","color":"b"}
            plot.add_line(xlim,[2,2],**kwargs)
            plot.add_line(xlim,[3.5,3.5],**kwargs)
            plot.add_line(xlim,[5,5],**kwargs)

            plot.ylim = [1.75,5.25]
            plot.xlabel = "Time [s]"
            plot.ylabel = "Velocity [km/s]"
            plot.show()
            plot.save(pngFile,dpi=200)
            plot.close()

def freq_analysis(params,channel,tt,freq,spectra):
    """@frequency analysis of spectral data.

    @param params
        seismon params dictionary
    @param channel
        seismon channel structure
    @param tt
        array of start times
    @param freq
        frequency vector
    @param spectra
        spectrogram structure
    """

    if params["doPlots"]:
        plotDirectory = params["path"] + "/" + channel.station_underscore + "/freq"
        seismon.utils.mkdir(plotDirectory)
 
    indexes = np.logspace(0,np.log10(len(freq)-1),num=100)
    indexes = list(np.unique(np.ceil(indexes)))
    indexes = range(len(freq))
    #indexes = range(16)

    indexes = np.where(10.0 >= freq)[0]

    deltaT = tt[1] - tt[0]

    n_dist = []
    for j in range(1000):
        n_dist.append(scipy.stats.chi2.rvs(2))

    p_chi2_vals = []
    p_ks_vals = []
    ttCoh_vals = []

    for i in indexes:
        vals = spectra[:,i]

        meanPSD = np.mean(vals) 
        stdPSD = np.std(vals)

        vals_norm = 2 * vals / meanPSD

        bins = np.arange(0,10,1)
        (n,bins) = np.histogram(vals_norm,bins=bins)
        n_total = np.sum(n)

        bins = (bins[1:] + bins[:len(bins)-1])/2

        n_expected = []
        for bin in bins:
            expected_val = n_total * scipy.stats.chi2.pdf(bin, 2)
            n_expected.append(expected_val)
        n_expected = np.array(n_expected)

        (stat_chi2,p_chi2) = scipy.stats.mstats.chisquare(n, f_exp=n_expected)
        p_chi2_vals.append(p_chi2)

        (stat_ks,p_ks) = scipy.stats.ks_2samp(vals_norm, n_dist)
        p_ks_vals.append(p_ks)

        acov = np.correlate(vals,vals,"full")
        acov = acov / np.max(acov)

        ttCov = (np.arange(len(acov)) - len(acov)/2) * float(deltaT)

        #ttLimitMin = - 5/freq[i]
        #ttLimitMax = 5 /freq[i]

        ttLimitMin = - float('inf')
        ttLimitMax = float('inf')

        ttIndexes = np.intersect1d(np.where(ttCov >= ttLimitMin)[0],np.where(ttCov <= ttLimitMax)[0])
        #ttCov = ttCov / (60)

        acov_minus_05 = np.absolute(acov[ttIndexes] - 0.66)
        index_min = acov_minus_05.argmin()

        ttCoh = np.absolute(ttCov[ttIndexes[index_min]])
        ttCoh_vals.append(ttCoh)

        if len(ttIndexes) == 0:
            continue

        #if freq[i] > 0:
        #    continue

        if params["doPlots"]:
            ax = plt.subplot(111)
            plt.plot(bins,n,label='true')
            plt.plot(bins,n_expected,'k*',label='expected')
            plt.xlabel("2 * data / mean")
            plt.ylabel("Counts")
            plot_title = "p-value: %f"%p_chi2
            plt.title(plot_title)
            plt.legend()
            plt.show()
            plt.savefig(os.path.join(plotDirectory,"%s_dist.png"%str(freq[i])),dpi=200)
            plt.savefig(os.path.join(plotDirectory,"%s_dist.eps"%str(freq[i])),dpi=200)
            plt.close('all')

            ax = plt.subplot(111)
            plt.semilogy(ttCov[ttIndexes],acov[ttIndexes])
            plt.vlines(ttCoh,10**(-3),1,color='r')
            plt.vlines(-ttCoh,10**(-3),1,color='r')
            plt.xlabel("Time [Seconds]")
            plt.ylabel("Correlation")
            plt.show()
            plt.savefig(os.path.join(plotDirectory,"%s_cov.png"%str(freq[i])),dpi=200)
            plt.savefig(os.path.join(plotDirectory,"%s_cov.eps"%str(freq[i])),dpi=200)
            plt.close('all')

    if params["doPlots"]:
        ax = plt.subplot(111)
        plt.loglog(freq[indexes],p_chi2_vals,label='chi2')
        plt.loglog(freq[indexes],p_ks_vals,label='k-s')
        plt.xlabel("Frequency [Hz]")
        plt.ylabel("p-value")
        plt.legend(loc=3)
        plt.show()
        plt.savefig(os.path.join(plotDirectory,"freq_analysis.png"),dpi=200)
        plt.savefig(os.path.join(plotDirectory,"freq_analysis.eps"),dpi=200)
        plt.close('all')      

        ax = plt.subplot(111)
        plt.semilogx(freq[indexes],ttCoh_vals)
        plt.xlabel("Frequency [Hz]")
        plt.ylabel("Coherence Time [s]")
        plt.show()
        plt.savefig(os.path.join(plotDirectory,"ttCohs.png"),dpi=200)
        plt.savefig(os.path.join(plotDirectory,"ttCohs.eps"),dpi=200)
        plt.close('all')

def analysis(params, channel):
    """@analysis of spectral data.

    @param params
        seismon params dictionary
    @param channel
        seismon channel structure
    """

    psdDirectory = params["dirPath"] + "/Text_Files/PSD/" + channel.station_underscore + "/" + str(params["fftDuration"])
    files = glob.glob(os.path.join(psdDirectory,"*.txt"))

    files = sorted(files)

    if not params["doFreqAnalysis"]:
        if len(files) > 1000:
            files = files[:1000]

    #files = files[-10:]

    tts = []
    spectra = []

    for file in files:

        fileSplit = file.split("/")
        txtFile = fileSplit[-1].replace(".txt","")
        txtFileSplit = txtFile.split("-")
        thisTTStart = int(txtFileSplit[0])
        thisTTEnd = int(txtFileSplit[1])
        tt = thisTTStart

        if tt in tts:
            continue

        tts.append(tt)

        spectra_out = gwpy.frequencyseries.Spectrum.read(file,format='dat')
        spectra_out.unit = 'counts/Hz^(1/2)'
        spectra.append(spectra_out)

        if tt == params["gpsStart"]:
            spectraNow = spectra_out

    if not 'spectraNow' in locals():
        print("no data at requested time... continuing\n")
        return

    if np.mean(spectraNow.value) == 0.0:
        print("data only zeroes... continuing\n")
        return

    dt = tts[1] - tts[0]
    epoch = gwpy.time.Time(tts[0], format='gps')
    specgram = gwpy.spectrogram.Spectrogram.from_spectra(*spectra, dt=dt,epoch=epoch)

    freq = np.array(specgram.frequencies)

    # Define bins for the spectral variation histogram
    kwargs = {'log':True,'nbins':500,'norm':True}
    #kwargs = {'log':True,'nbins':500}
    specvar = gwpy.frequencyseries.hist.SpectralVariance.from_spectrogram(specgram,**kwargs) 
    bins = specvar.bins[:-1]
    specvar = specvar * 100

    if params["doFreqAnalysis"]:
        freq_analysis(params,channel,ttStart,freq,specgram)

    # Calculate percentiles
    spectral_variation_1per = specvar.percentile(1)
    spectral_variation_10per = specvar.percentile(10)
    spectral_variation_50per = specvar.percentile(50)
    spectral_variation_90per = specvar.percentile(90)
    spectral_variation_99per = specvar.percentile(99)

    textDirectory = params["path"] + "/" + channel.station_underscore
    seismon.utils.mkdir(textDirectory)

    f = open(os.path.join(textDirectory,"spectra.txt"),"w")
    for i in range(len(freq)):
        f.write("%e %e %e %e %e %e %e\n"%(freq[i],spectral_variation_1per[i].value,\
            spectral_variation_10per[i].value,spectral_variation_50per[i].value,\
            spectral_variation_90per[i].value,spectral_variation_99per[i].value,\
            spectraNow[i].value))
    f.close()

    sigDict = {}
    # Break up entire frequency band into 6 segments
    ff_ave = [1/float(128), 1/float(64),  0.1, 1, 3, 5, 10]

    f = open(os.path.join(textDirectory,"sig.txt"),"w")
    for i in range(len(ff_ave)-1):
        newSpectra = []
        newSpectraNow = []
        newFreq = []

        for j in range(len(freq)):
            if ff_ave[i] <= freq[j] and freq[j] <= ff_ave[i+1]:
                newFreq.append(freq[j])
                newSpectraNow.append(spectraNow.value[j])
                if newSpectra == []:
                    newSpectra = specgram.value[:,j]
                else:                 
                    newSpectra = np.vstack([newSpectra,specgram.value[:,j]])

        newSpectra = np.array(newSpectra)
        if len(newSpectra.shape) > 1:
            newSpectra = np.mean(newSpectra, axis = 0)
        try:
            sig, bgcolor = seismon.utils.html_bgcolor(np.mean(newSpectraNow),newSpectra)
        except:
            sig = 0
            bgcolor = 'nan'

        f.write("%e %e %e %e %s\n"%(ff_ave[i],ff_ave[i+1],np.mean(newSpectraNow),sig,bgcolor))

        key = "%s-%s"%(ff_ave[i],ff_ave[i+1])

        dt = tts[-1] - tts[-2]
        epoch = gwpy.time.Time(tts[0], format='gps')

        timeseries = gwpy.timeseries.TimeSeries(newSpectra, epoch=epoch, sample_rate=1.0/dt)
        
        sigDict[key] = {}
        sigDict[key]["data"] = timeseries

    f.close()

    if params["doPlots"]:

        plotDirectory = params["path"] + "/" + channel.station_underscore
        seismon.utils.mkdir(plotDirectory)

        fl, low, fh, high = seismon.NLNM.NLNM(2)

        pngFile = os.path.join(plotDirectory,"psd.png")

        #plot = spectraNow.plot()
        plot = gwpy.plotter.Plot(figsize=[14,8])
        plot.add_spectrum(spectraNow,label='now')
        kwargs = {"linestyle":"-","color":"k"}
        plot.add_line(freq, spectral_variation_10per, **kwargs)
        plot.add_line(freq, spectral_variation_50per, **kwargs)
        plot.add_line(freq, spectral_variation_90per, **kwargs)
        kwargs = {"linestyle":"-.","color":"k"}
        plot.add_line(fl, low, **kwargs)
        plot.add_line(fh, high, **kwargs)
        plot.xlim = [params["fmin"],params["fmax"]]
        plot.ylim = [np.min(bins), np.max(bins)]
        plot.xlabel = "Frequency [Hz]"
        plot.ylabel = "Amplitude Spectrum [(m/s)/rtHz]"
        plot.axes[0].set_xscale("log")
        plot.axes[0].set_yscale("log")
        plot.save(pngFile,dpi=200)
        plot.close()

        pngFile = os.path.join(plotDirectory,"disp.png")

        spectraNowDisplacement = spectraNow / freq
        #plot = spectraNowDisplacement.plot()

        plot = gwpy.plotter.Plot(figsize=[14,8])
        plot.add_spectrum(spectraNowDisplacement,label='now')
        kwargs = {"linestyle":"-","color":"w"}
        plot.add_line(freq, spectral_variation_10per/freq, **kwargs)
        plot.add_line(freq, spectral_variation_50per/freq, **kwargs)
        plot.add_line(freq, spectral_variation_90per/freq, **kwargs)
        kwargs = {"linestyle":"-.","color":"k"}
        plot.add_line(fl, low/fl, **kwargs)
        plot.add_line(fh, high/fh, **kwargs)
        plot.xlim = [params["fmin"],params["fmax"]]
        plot.ylim = [np.min(bins)/np.max(freq), np.max(bins)/np.min(freq)]
        plot.xlabel = "Frequency [Hz]"
        plot.ylabel = "Displacement Spectrum [m/rtHz]"
        plot.axes[0].set_xscale("log")
        plot.axes[0].set_yscale("log")
        plot.save(pngFile,dpi=200)
        plot.close()

        pngFile = os.path.join(plotDirectory,"tf.png")
        #specgramLog = specgram.to_logf(fmin=np.min(freq),fmax=np.max(freq))
        #plot = specgramLog.plot()
        plot = specgram.plot()
        plot.ylim = [params["fmin"],params["fmax"]]
        plot.ylabel = "Frequency [Hz]"
        colorbar_label = "Amplitude Spectrum [(m/s)/rtHz]"
        kwargs = {}
        plot.axes[0].set_xscale("log")
        plot.axes[0].set_yscale("log")
        plot.add_colorbar(location='right', log=True, label=colorbar_label, clim=None, visible=True, **kwargs)
        plot.save(pngFile,dpi=200)
        plot.close()

        pngFile = os.path.join(plotDirectory,"specvar.png")
        kwargs = {"linestyle":"-","color":"w"}
        #plot = specvar.plot(**kwargs)
        plot = gwpy.plotter.Plot(figsize=[14,8])
        plot.add_spectrum(spectraNow,label='now')
        #plot = spectraNow.plot(**kwargs)
        kwargs = {"linestyle":"-","color":"k"}
        plot.add_line(freq, spectral_variation_10per, **kwargs)
        plot.add_line(freq, spectral_variation_50per, **kwargs)
        plot.add_line(freq, spectral_variation_90per, **kwargs)
        extent = [np.min(freq), np.max(freq),
                   np.min(bins), np.max(bins)]
        kwargs = {}
        #plot.plot_variance(specvar, extent=extent, **kwargs)
        plot.axes[0].set_xscale("log")
        plot.axes[0].set_yscale("log")
        kwargs = {"linestyle":"-.","color":"k"}
        plot.add_line(fl, low, **kwargs)
        plot.add_line(fh, high, **kwargs)
        plot.xlim = [params["fmin"],params["fmax"]]
        plot.ylim = [np.min(bins), np.max(bins)]
        plot.xlabel = "Frequency [Hz]"
        plot.ylabel = "Amplitude Spectrum [(m/s)/rtHz]"

        plot.save(pngFile,dpi=200)
        plot.close()

        X,Y = np.meshgrid(freq, bins)
        ax = plt.subplot(111)
        #im = plt.pcolor(X,Y,np.transpose(spectral_variation_norm), cmap=plt.cm.jet)

        im = plt.pcolor(X,Y,np.transpose(specvar.value), cmap=plt.cm.jet)
        ax.set_xscale('log')
        ax.set_yscale('log')
        plt.semilogx(freq,spectraNow, 'k', label='Current')
        plt.semilogx(freq,spectral_variation_10per,'w',label='10')
        plt.semilogx(freq,spectral_variation_50per,'w',label='50')
        plt.semilogx(freq,spectral_variation_90per,'w',label='90')
        plt.loglog(fl,low,'k-.')
        plt.loglog(fh,high,'k-.',label='LNM/HNM')
        plt.xlim([params["fmin"],params["fmax"]])
        plt.ylim([np.min(bins), np.max(bins)])
        plt.xlabel("Frequency [Hz]")
        plt.ylabel("Amplitude Spectrum [(m/s)/rtHz]")
        plt.clim(0,5)
        plt.grid()
        plt.show()
        plt.savefig(pngFile,dpi=200)
        plt.close('all')

        pngFile = os.path.join(plotDirectory,"bands.png")
        plot = gwpy.plotter.TimeSeriesPlot()
        for key in sigDict.iterkeys():
            label = key
            plot.add_timeseries(sigDict[key]["data"], label=label)
        plot.axes[0].set_yscale("log")
        plot.ylabel = "Average Amplitude Spectrum log10[(m/s)/rtHz]"
        plot.add_legend(loc=1,prop={'size':10})
        plot.save(pngFile,dpi=200)
        plot.close()

    htmlPage = seismon.html.seismon_page(channel,textDirectory)
    if htmlPage is not None:
        f = open(os.path.join(textDirectory,"psd.html"),"w")
        f.write(htmlPage)
        f.close()

def channel_summary(params, segment):
    """@summary of channels of spectral data.

    @param params
        seismon params dictionary
    @param segment
        [start,end] gps
    """

    gpsStart = segment[0]
    gpsEnd = segment[1]

    data = {}
    for channel in params["channels"]:

        psdDirectory = params["dirPath"] + "/Text_Files/PSD/" + channel.station_underscore + "/" + str(params["fftDuration"])
        file = os.path.join(psdDirectory,"%d-%d.txt"%(gpsStart,gpsEnd))

        if not os.path.isfile(file):
            continue

        spectra_out = gwpy.frequencyseries.Spectrum.read(file,format='dat')
        spectra_out.unit = 'counts/Hz^(1/2)'

        if np.sum(spectra_out.value) == 0.0:
            continue

        data[channel.station_underscore] = {}
        data[channel.station_underscore]["data"] = spectra_out

        if params["doPowerLawFit"]:

            powerlawDirectory = params["dirPath"] + "/Text_Files/Powerlaw/" + channel.station_underscore + "/" + str(params["fftDuration"])
            seismon.utils.mkdir(powerlawDirectory)

            powerlawFile = os.path.join(powerlawDirectory,"%d-%d.txt"%(gpsStart,gpsEnd))
            data_out = np.loadtxt(powerlawFile)
            data[channel.station_underscore]["powerlaw"] = data_out

    if data == {}:
        return

    if params["doPlots"]:

        plotDirectory = params["path"] + "/summary"
        seismon.utils.mkdir(plotDirectory)

        fl, low, fh, high = seismon.NLNM.NLNM(2)

        pngFile = os.path.join(plotDirectory,"psd.png")
        lowBin = np.inf
        highBin = -np.inf
        plot = gwpy.plotter.Plot(figsize=[14,8])
        for key in data.iterkeys():

            label = key.replace("_","\_")

            plot.add_spectrum(data[key]["data"], label=label)

            lowBin = np.min([lowBin,np.min(np.array(data[key]["data"]))])
            highBin = np.max([highBin,np.max(np.array(data[key]["data"]))])

        kwargs = {"linestyle":"-.","color":"k"}
        plot.add_line(fl, low, **kwargs)
        plot.add_line(fh, high, **kwargs)
        plot.xlim = [params["fmin"],params["fmax"]]
        plot.ylim = [lowBin, highBin]
        plot.xlabel = "Frequency [Hz]"
        plot.ylabel = "Amplitude Spectrum [(m/s)/rtHz]"
        plot.add_legend(loc=1,prop={'size':10})
        plot.axes[0].set_xscale("log")
        plot.axes[0].set_yscale("log")

        plot.save(pngFile,dpi=200)
        plot.close()

        pngFile = os.path.join(plotDirectory,"ratio.png")
        lowBin = np.inf
        highBin = -np.inf
        ref = params["referenceChannel"].replace(":","_")

        if not ref in data:
            return

        plot = gwpy.plotter.Plot(figsize=[14,8])
        for key in data.iterkeys():

            label = key.replace("_","\_")

            plot.add_spectrum(data[key]["data"] / data[ref]["data"], label=label)
            lowBin = np.min([lowBin,np.min(np.array(data[key]["data"]))])
            highBin = np.max([highBin,np.max(np.array(data[key]["data"]))])

        kwargs = {"linestyle":"-.","color":"k"}
        #plot.add_line(fl, low, **kwargs)
        #plot.add_line(fh, high, **kwargs)
        plot.xlim = [params["fmin"],params["fmax"]]
        #plot.ylim = [lowBin, highBin]
        plot.xlabel = "Frequency [Hz]"
        label_ref = params["referenceChannel"].replace("_","\_")
        plot.ylabel = "Spectrum / Reference [%s]"%(label_ref)
        plot.add_legend(loc=1,prop={'size':10})
        plot.axes[0].set_xscale("log")
        plot.axes[0].set_yscale("log")

        plot.save(pngFile,dpi=200)
        plot.close()

        if params["doPowerLawFit"]:

            pngFile = os.path.join(plotDirectory,"powerlaw.png")
            colors = cm.rainbow(np.linspace(0, 1, len(data)))
            count = 0

            plot = gwpy.plotter.Plot(figsize=[14,8])
            for key in data.iterkeys():

                label = key.replace("_","\_")

                color = colors[count]
                kwargs = {"linestyle":"-","color":color}

                plot.add_spectrum(data[key]["data"], label=label, **kwargs)

                powerlaw = lambda x, amp, index: amp * (x**index)

                xdata = data[key]["data"].frequencies
                index = data[key]["powerlaw"][0]
                amp = data[key]["powerlaw"][1]

                kwargs = {"linestyle":"--","color":color}
                plot.add_line(xdata,powerlaw(xdata,amp,index),**kwargs)
                count = count + 1

            kwargs = {"linestyle":"-.","color":"k"}
            plot.add_line(fl, low, **kwargs)
            plot.add_line(fh, high, **kwargs)
            plot.xlim = [params["fmin"],params["fmax"]]
            plot.ylim = [lowBin, highBin]
            plot.xlabel = "Frequency [Hz]"
            plot.ylabel = "Amplitude Spectrum [(m/s)/rtHz]"
            plot.add_legend(loc=1,prop={'size':10})
            plot.axes[0].set_xscale("log")
            plot.axes[0].set_yscale("log")

            plot.save(pngFile,dpi=200)
            plot.close()




#!/usr/bin/python

import sys, os, glob
import numpy as np
import scipy.linalg

import seismon.utils

try:
    import gwpy.time, gwpy.timeseries
    import gwpy.frequencyseries, gwpy.spectrogram
except:
    print("gwpy import fails... no plotting possible.")


__author__ = "Michael Coughlin <michael.coughlin@ligo.org>"
__date__ = "2012/8/26"
__version__ = "0.1"

# =============================================================================
#
#                               DEFINITIONS
#
# =============================================================================

def wiener(params, target_channel, segment):
    """@calculates wiener filter for given channel and segment.

    @param params
        seismon params dictionary
    @param target_channel
        seismon channel structure
    @param segment
        [start,end] gps
    """

    ifo = seismon.utils.getIfo(params)

    gpsStart = segment[0]
    gpsEnd = segment[1]

    # set the times
    duration = np.ceil(gpsEnd-gpsStart)

    dataAll = []

    if params["wienerFilterSampleRate"] > 0:
        samplef =params["wienerFilterSampleRate"]
    else:
        samplef = target_channel.samplef
    N = params["wienerFilterOrder"]

    for channel in params["channels"]:
        # make timeseries
        dataFull = seismon.utils.retrieve_timeseries(params, channel, segment)
        if dataFull == []:
            continue

        dataFull = dataFull / channel.calibration
        indexes = np.where(np.isnan(dataFull.data))[0]
        meanSamples = np.mean(np.ma.masked_array(dataFull.data,np.isnan(dataFull.data)))
        for index in indexes:
            dataFull[index] = meanSamples
        dataFull -= np.mean(dataFull.data)

        if np.mean(dataFull.data) == 0.0:
            print("data only zeroes... continuing\n")
            continue
        if len(dataFull.data) < 2*channel.samplef:
            print("timeseries too short for analysis... continuing\n")
            continue

        if params["wienerFilterSampleRate"] > 0:
            dataFull = dataFull.resample(params["wienerFilterSampleRate"])

        if (params["wienerFilterLowFreq"] > 0) and (params["wienerFilterHighFreq"] == 0):
            dataFull = dataFull.highpass(params["wienerFilterLowFreq"], amplitude=0.9, order=3, method='scipy')
        elif (params["wienerFilterLowFreq"] == 0) and (params["wienerFilterHighFreq"] > 0):
            dataFull = dataFull.lowpass(params["wienerFilterHighFreq"], amplitude=0.9, order=3, method='scipy')
        elif (params["wienerFilterLowFreq"] > 0) and (params["wienerFilterHighFreq"] > 0):
            dataFull = dataFull.bandpass(params["wienerFilterLowFreq"],params["wienerFilterHighFreq"], amplitude=0.9, order=3, method='scipy')

        indexes = np.where(np.isnan(dataFull.data))[0]
        meanSamples = np.mean(np.ma.masked_array(dataFull.data,np.isnan(dataFull.data)))
        for index in indexes:
            dataFull[index] = meanSamples
        dataFull -= np.mean(dataFull.data)

        dataAll.append(dataFull)

    X = []
    y = []
    for dataFull in dataAll:
        if dataFull.channel.name == target_channel.station:
            tt = np.array(dataFull.times)
            y = dataFull.data
        else:
            if X == []:
                X = dataFull.data
            else:
                try:
                    X = np.vstack([X,dataFull.data])
                except:
                    continue

    if len(y) == 0:
        print("No data for target channel... continuing")
        return

    originalASD = []
    residualASD = []
    FFASD = []

    gpss = np.arange(gpsStart,gpsEnd,params["fftDuration"])
    create_filter = True
    for i in range(len(gpss)-1):
        tt = np.array(dataFull.times)
        indexes = np.intersect1d(np.where(tt >= gpss[i])[0],np.where(tt <= gpss[i+1]+N)[0])

        if len(indexes) == 0:
            continue

        indexMin = np.min(indexes)
        indexMax = np.max(indexes)

        ttCut = tt[indexMin:indexMax] 
        yCut = y[indexMin:indexMax]

        if len(X.shape) == 1:
            XCut = np.reshape(X[indexMin:indexMax],(-1,1))
        else:
            XCut = X[:,indexMin:indexMax]
            XCut = XCut.T
        if create_filter:
            print("Generating filter")
            W,R,P = miso_firwiener(N,XCut,yCut)
            create_filter = False
            print("finished generating filter")
            continue
            
        residual, FF = subtractFF(W,XCut,yCut,samplef)

        thisGPSStart = tt[indexMin]
        dataOriginal = gwpy.timeseries.TimeSeries(yCut, epoch=thisGPSStart, sample_rate=samplef,name="Original")
        dataResidual = gwpy.timeseries.TimeSeries(residual, epoch=thisGPSStart, sample_rate=samplef,name="Residual")
        dataFF = gwpy.timeseries.TimeSeries(FF, epoch=thisGPSStart, sample_rate=samplef,name="FF")

        # calculate spectrum
        NFFT = params["fftDuration"]
        #window = None
        dataOriginalASD = dataOriginal.asd(NFFT,NFFT,'welch')
        dataResidualASD = dataResidual.asd(NFFT,NFFT,'welch')
        dataFFASD = dataFF.asd(NFFT,NFFT,'welch')

        freq = np.array(dataOriginalASD.frequencies)
        indexes = np.where((freq >= params["fmin"]) & (freq <= params["fmax"]))[0]
        freq = freq[indexes]

        dataOriginalASD = np.array(dataOriginalASD.data)
        dataOriginalASD = dataOriginalASD[indexes]
        dataOriginalASD = gwpy.frequencyseries.Spectrum(dataOriginalASD, f0=np.min(freq), df=(freq[1]-freq[0]))

        dataResidualASD = np.array(dataResidualASD.data)
        dataResidualASD = dataResidualASD[indexes]
        dataResidualASD = gwpy.frequencyseries.Spectrum(dataResidualASD, f0=np.min(freq), df=(freq[1]-freq[0]))

        dataFFASD = np.array(dataFFASD.data)
        dataFFASD = dataFFASD[indexes]
        dataFFASD = gwpy.frequencyseries.Spectrum(dataFFASD, f0=np.min(freq), df=(freq[1]-freq[0]))

        originalASD.append(dataOriginalASD)
        residualASD.append(dataResidualASD)
        FFASD.append(dataFFASD)

    dt = gpss[1] - gpss[0]
    epoch = gwpy.time.Time(gpss[0], format='gps')
    originalSpecgram = gwpy.spectrogram.Spectrogram.from_spectra(*originalASD, dt=dt,epoch=epoch)
    residualSpecgram = gwpy.spectrogram.Spectrogram.from_spectra(*residualASD, dt=dt,epoch=epoch)
    FFSpecgram = gwpy.spectrogram.Spectrogram.from_spectra(*FFASD, dt=dt,epoch=epoch)

    freq = np.array(originalSpecgram.frequencies)

    kwargs = {'log':True,'nbins':500,'norm':True}
    originalSpecvar = gwpy.frequencyseries.hist.SpectralVariance.from_spectrogram(originalSpecgram,**kwargs)
    bins = originalSpecvar.bins[:-1]
    originalSpecvar = originalSpecvar * 100
    original_spectral_variation_50per = originalSpecvar.percentile(50)
    residualSpecvar = gwpy.frequencyseries.hist.SpectralVariance.from_spectrogram(residualSpecgram,**kwargs)
    bins = residualSpecvar.bins[:-1]
    residualSpecvar = residualSpecvar * 100
    residual_spectral_variation_50per = residualSpecvar.percentile(50)
    FFSpecvar = gwpy.frequencyseries.hist.SpectralVariance.from_spectrogram(FFSpecgram,**kwargs)
    bins = FFSpecvar.bins[:-1]
    FFSpecvar = FFSpecvar * 100
    FF_spectral_variation_50per = FFSpecvar.percentile(50)

    psdDirectory = params["dirPath"] + "/Text_Files/Wiener/" + target_channel.station_underscore + "/" + str(params["fftDuration"]) + "/" + str(params["wienerFilterOrder"])
    seismon.utils.mkdir(psdDirectory)

    freq = np.array(residual_spectral_variation_50per.frequencies)

    psdFile = os.path.join(psdDirectory,"%d-%d.txt"%(gpsStart,gpsEnd))
    f = open(psdFile,"wb")
    for i in range(len(freq)):
        f.write("%e %e\n"%(freq[i],residual_spectral_variation_50per[i]))
    f.close()

    if params["doPlots"]:

        plotDirectory = params["path"] + "/Wiener/" + target_channel.station_underscore + "/" + str(params["fftDuration"]) + "/" + str(params["wienerFilterOrder"])
        seismon.utils.mkdir(plotDirectory)

        fl, low, fh, high = seismon.NLNM.NLNM(2)

        pngFile = os.path.join(plotDirectory,"psd.png")

        plot = gwpy.plotter.Plot(figsize=[14,8])
        kwargs = {"linestyle":"-","color":"b","label":"Original"}
        plot.add_line(freq, original_spectral_variation_50per, **kwargs)
        kwargs = {"linestyle":"-","color":"g","label":"Residual"}
        plot.add_line(freq, residual_spectral_variation_50per, **kwargs)
        kwargs = {"linestyle":"-","color":"r","label":"FF"}
        plot.add_line(freq, FF_spectral_variation_50per, **kwargs)
        kwargs = {"linestyle":"-.","color":"k"}
        plot.add_line(fl, low, **kwargs)
        plot.add_line(fh, high, **kwargs)
        plot.axes[0].set_yscale("log")
        plot.axes[0].set_xscale("log")
        plot.xlim = [params["fmin"],params["fmax"]]
        #plot.ylim = [np.min(bins), np.max(bins)]
        plot.xlabel = "Frequency [Hz]"
        plot.ylabel = "Amplitude Spectrum [(m/s)/rtHz]"
        plot.add_legend(loc=1,prop={'size':10})
        plot.save(pngFile,dpi=200)
        plot.close()

        pngFile = os.path.join(plotDirectory,"ratio.png")

        plot = gwpy.plotter.Plot(figsize=[14,8])
        kwargs = {"linestyle":"-","color":"b"}
        plot.add_line(freq, residual_spectral_variation_50per/original_spectral_variation_50per, **kwargs)
        plot.axes[0].set_xscale("log")
        plot.xlim = [params["fmin"],params["fmax"]]
        #plot.ylim = [np.min(bins), np.max(bins)]
        plot.xlabel = "Frequency [Hz]"
        plot.ylabel = "Amplitude Ratio"
        plot.save(pngFile,dpi=200)
        plot.close()

def wiener_hilbert(params, segment):
    """@calculates wiener filter for given channel and segment.

    @param params
        seismon params dictionary
    @param target_channel
        seismon channel structure
    @param segment
        [start,end] gps
    """

    ifo = seismon.utils.getIfo(params)

    gpsStart = segment[0]
    gpsEnd = segment[1]

    # set the times
    duration = np.ceil(gpsEnd-gpsStart)

    dataAll = []

    if params["wienerFilterSampleRate"] > 0:
        samplef =params["wienerFilterSampleRate"]
    else:
        samplef = params["channels"][0].samplef
    N = params["wienerFilterOrder"]

    for channel in params["channels"]:
        # make timeseries
        dataFull = seismon.utils.retrieve_timeseries(params, channel, segment)
        if dataFull == []:
            continue

        dataFull = dataFull / channel.calibration
        indexes = np.where(np.isnan(dataFull.data))[0]
        meanSamples = np.mean(np.ma.masked_array(dataFull.data,np.isnan(dataFull.data)))
        for index in indexes:
            dataFull[index] = meanSamples
        dataFull -= np.mean(dataFull.data)

        if np.mean(dataFull.data) == 0.0:
            print("data only zeroes... continuing\n")
            continue
        if len(dataFull.data) < 2*channel.samplef:
            print("timeseries too short for analysis... continuing\n")
            continue

        if params["wienerFilterSampleRate"] > 0:
            dataFull = dataFull.resample(samplef)

        if (params["wienerFilterLowFreq"] > 0) and (params["wienerFilterHighFreq"] == 0):
            dataFull = dataFull.highpass(params["wienerFilterLowFreq"], amplitude=0.9, order=3, method='scipy')
        elif (params["wienerFilterLowFreq"] == 0) and (params["wienerFilterHighFreq"] > 0):
            dataFull = dataFull.lowpass(params["wienerFilterHighFreq"], amplitude=0.9, order=3, method='scipy')
        elif (params["wienerFilterLowFreq"] > 0) and (params["wienerFilterHighFreq"] > 0):
            dataFull = dataFull.bandpass(params["wienerFilterLowFreq"],params["wienerFilterHighFreq"], amplitude=0.9, order=3, method='scipy')

        indexes = np.where(np.isnan(dataFull.data))[0]
        meanSamples = np.mean(np.ma.masked_array(dataFull.data,np.isnan(dataFull.data)))
        for index in indexes:
            dataFull[index] = meanSamples
        dataFull -= np.mean(dataFull.data)

        dataAll.append(dataFull)

    for dataFull in dataAll:
        if "X" in dataFull.channel.name or "E" == dataFull.channel.name[-1]:
            tsx = dataFull.data
        if "Y" in dataFull.channel.name or "N" == dataFull.channel.name[-1]:
            tsy = dataFull.data
        if "Z" in dataFull.channel.name:
            tsz = dataFull.data

    tt = np.array(dataAll[0].times)

    tszhilbert = scipy.signal.hilbert(tsz).imag
    tszhilbert = -tszhilbert

    angles = np.linspace(0,2*np.pi,60)
    xcorrs = []
    for angle in angles:
        rot = np.array([[np.cos(angle), -np.sin(angle)],[np.sin(angle),np.cos(angle)]])
    
        twodarray = np.vstack([tsx,tsy])
        z = rot.dot(twodarray)
        tsxy = np.sum(z.T,axis=1)
    
        tt = np.array(dataFull.times)
        indexes = np.arange(0,len(tt))

        indexMin = np.min(indexes)
        indexMax = np.max(indexes)

        tszhilbertCut = tszhilbert[indexMin:indexMax]
        tsxyCut = tsxy[indexMin:indexMax]

        xcorr,lags = seismon.utils.xcorr(tszhilbertCut,tsxyCut,maxlags=1)
        xcorrs.append(xcorr[1])
    xcorrs = np.array(xcorrs)

    angleMax = angles[np.argmax(xcorrs)]
    rot = np.array([[np.cos(angleMax), -np.sin(angleMax)],[np.sin(angleMax),np.cos(angleMax)]])

    print("Using angle: %f"%(angleMax))

    #angle = 0
    #rot = np.array([[np.cos(angle), -np.sin(angle)],[np.sin(angle),np.cos(angle)]])

    twodarray = np.vstack([tsx,tsy])
    z = rot.dot(twodarray)
    tsxy = np.sum(z.T,axis=1)

    dataHilbert = tszhilbert.view(tsz.__class__)
    dataHilbert = gwpy.timeseries.TimeSeries(dataHilbert)
    dataHilbert.sample_rate = samplef 
    dataHilbert.epoch = tt[0]
    dataHilbert = dataHilbert.resample(1)

    dataXY = tsxy.view(tsz.__class__)
    dataXY = gwpy.timeseries.TimeSeries(tsxy)
    dataXY.sample_rate = samplef
    dataXY.epoch = tt[0]
    dataXY = dataXY.resample(1)

    dataZ = gwpy.timeseries.TimeSeries(tsz)
    dataZ.sample_rate = samplef
    dataZ.epoch = tt[0]
    dataZ = dataZ.resample(1)

    if params["doPlots"]:

        plotDirectory = params["path"] + "/Wiener_Hilbert/" + params["channels"][0].station_underscore + "/" + str(params["fftDuration"]) + "/" + str(params["wienerFilterOrder"])
        seismon.utils.mkdir(plotDirectory)

        pngFile = os.path.join(plotDirectory,"timeseries.png")

        dataHilbert *= 1e6
        dataXY *= 1e6
        dataZ *= 1e6

        plot = gwpy.plotter.TimeSeriesPlot(figsize=[14,8])

        kwargs = {"linestyle":"-","color":"b"}
        plot.add_timeseries(dataHilbert,label="Hilbert",**kwargs)
        kwargs = {"linestyle":"-","color":"g"}
        plot.add_timeseries(dataXY,label="XY",**kwargs)
        kwargs = {"linestyle":"-","color":"r"}
        #plot.add_timeseries(dataZ,label="Z",**kwargs)
        plot.ylabel = r"Velocity [$\mu$m/s]"
        plot.add_legend(loc=1,prop={'size':10})

        plot.save(pngFile)
        plot.close()

    y = tsxy
    X = np.vstack([tszhilbert,tsz])

    if len(y) == 0:
        print("No data for target channel... continuing")
        return

    originalASD = []
    residualASD = []
    FFASD = []

    gpss = np.arange(gpsStart,gpsEnd,params["fftDuration"])
    create_filter = True
    for i in range(len(gpss)-2):
        tt = np.array(dataFull.times)
        indexes = np.intersect1d(np.where(tt >= gpss[i])[0],np.where(tt <= gpss[i+1]+params["fftDuration"])[0])

        if len(indexes) == 0:
            continue

        indexMin = np.min(indexes)
        indexMax = np.max(indexes)

        ttCut = tt[indexMin:indexMax]
        yCut = y[indexMin:indexMax]
        XCut = X[:,indexMin:indexMax]

        XCut = XCut.T
        if create_filter:
            print("Generating filter")
            W,R,P = miso_firwiener(N,XCut,yCut)
            create_filter = False
            print("finished generating filter")
            continue

        residual, FF = subtractFF(W,XCut,yCut,samplef)

        thisGPSStart = tt[indexMin]
        dataOriginal = gwpy.timeseries.TimeSeries(yCut, epoch=thisGPSStart, sample_rate=samplef,name="Original")
        dataResidual = gwpy.timeseries.TimeSeries(residual, epoch=thisGPSStart, sample_rate=samplef,name="Residual")
        dataFF = gwpy.timeseries.TimeSeries(FF, epoch=thisGPSStart, sample_rate=samplef,name="FF")

        # calculate spectrum
        NFFT = params["fftDuration"]
        #window = None
        dataOriginalASD = dataOriginal.asd(NFFT,NFFT,'welch')
        dataResidualASD = dataResidual.asd(NFFT,NFFT,'welch')
        dataFFASD = dataFF.asd(NFFT,NFFT,'welch')

        freq = np.array(dataOriginalASD.frequencies)
        indexes = np.where((freq >= params["fmin"]) & (freq <= params["fmax"]))[0]
        freq = freq[indexes]

        dataOriginalASD = np.array(dataOriginalASD.data)
        dataOriginalASD = dataOriginalASD[indexes]
        dataOriginalASD = gwpy.frequencyseries.Spectrum(dataOriginalASD, f0=np.min(freq), df=(freq[1]-freq[0]))

        dataResidualASD = np.array(dataResidualASD.data)
        dataResidualASD = dataResidualASD[indexes]
        dataResidualASD = gwpy.frequencyseries.Spectrum(dataResidualASD, f0=np.min(freq), df=(freq[1]-freq[0]))

        dataFFASD = np.array(dataFFASD.data)
        dataFFASD = dataFFASD[indexes]
        dataFFASD = gwpy.frequencyseries.Spectrum(dataFFASD, f0=np.min(freq), df=(freq[1]-freq[0]))

        originalASD.append(dataOriginalASD)
        residualASD.append(dataResidualASD)
        FFASD.append(dataFFASD)

    dt = gpss[1] - gpss[0]
    epoch = gwpy.time.Time(gpss[0], format='gps')
    originalSpecgram = gwpy.spectrogram.Spectrogram.from_spectra(*originalASD, dt=dt,epoch=epoch)
    residualSpecgram = gwpy.spectrogram.Spectrogram.from_spectra(*residualASD, dt=dt,epoch=epoch)
    FFSpecgram = gwpy.spectrogram.Spectrogram.from_spectra(*FFASD, dt=dt,epoch=epoch)

    freq = np.array(originalSpecgram.frequencies)

    kwargs = {'log':True,'nbins':500,'norm':True}
    originalSpecvar = gwpy.frequencyseries.hist.SpectralVariance.from_spectrogram(originalSpecgram,**kwargs)
    bins = originalSpecvar.bins[:-1]
    originalSpecvar = originalSpecvar * 100
    original_spectral_variation_50per = originalSpecvar.percentile(50)
    residualSpecvar = gwpy.frequencyseries.hist.SpectralVariance.from_spectrogram(residualSpecgram,**kwargs)
    bins = residualSpecvar.bins[:-1]
    residualSpecvar = residualSpecvar * 100
    residual_spectral_variation_50per = residualSpecvar.percentile(50)
    FFSpecvar = gwpy.frequencyseries.hist.SpectralVariance.from_spectrogram(FFSpecgram,**kwargs)
    bins = FFSpecvar.bins[:-1]
    FFSpecvar = FFSpecvar * 100
    FF_spectral_variation_50per = FFSpecvar.percentile(50)

    psdDirectory = params["dirPath"] + "/Text_Files/Wiener_Hilbert/" + params["channels"][0].station_underscore + "/" + str(params["fftDuration"]) + "/" + str(params["wienerFilterOrder"])
    seismon.utils.mkdir(psdDirectory)

    freq = np.array(residual_spectral_variation_50per.frequencies)

    psdFile = os.path.join(psdDirectory,"%d-%d.txt"%(gpsStart,gpsEnd))
    f = open(psdFile,"wb")
    for i in range(len(freq)):
        f.write("%e %e\n"%(freq[i],residual_spectral_variation_50per[i]))
    f.close()

    if params["doPlots"]:

        plotDirectory = params["path"] + "/Wiener_Hilbert/" + params["channels"][0].station_underscore + "/" + str(params["fftDuration"]) + "/" + str(params["wienerFilterOrder"])
        seismon.utils.mkdir(plotDirectory)

        fl, low, fh, high = seismon.NLNM.NLNM(2)

        pngFile = os.path.join(plotDirectory,"psd.png")

        plot = gwpy.plotter.Plot(figsize=[14,8])
        kwargs = {"linestyle":"-","color":"b","label":"Original"}
        plot.add_line(freq, original_spectral_variation_50per, **kwargs)
        kwargs = {"linestyle":"-","color":"g","label":"Residual"}
        plot.add_line(freq, residual_spectral_variation_50per, **kwargs)
        kwargs = {"linestyle":"-","color":"r","label":"FF"}
        plot.add_line(freq, FF_spectral_variation_50per, **kwargs)
        kwargs = {"linestyle":"-.","color":"k"}
        plot.add_line(fl, low, **kwargs)
        plot.add_line(fh, high, **kwargs)
        plot.axes[0].set_yscale("log")
        plot.axes[0].set_xscale("log")
        plot.xlim = [params["fmin"],params["fmax"]]
        #plot.ylim = [np.min(bins), np.max(bins)]
        plot.xlabel = "Frequency [Hz]"
        plot.ylabel = "Amplitude Spectrum [(m/s)/rtHz]"
        plot.add_legend(loc=1,prop={'size':10})
        plot.save(pngFile,dpi=200)
        plot.close()

def miso_firwiener(N,X,y):

    # MISO_FIRWIENER Optimal FIR Wiener filter for multiple inputs.
    # MISO_FIRWIENER(N,X,Y) computes the optimal FIR Wiener filter of order
    # N, given any number of (stationary) random input signals as the columns
    # of matrix X, and one output signal in column vector Y.
    # Author: Keenan Pepper
    # Last modified: 2007/08/02
    # References:
    # [1] Y. Huang, J. Benesty, and J. Chen, Acoustic MIMO Signal
    # Processing, SpringerVerlag, 2006, page 48

    # Number of input channels.
    try:
        junk, M = X.shape
    except:
        M = 1

    # Input covariance matrix.
    R = np.zeros([M*(N+1),M*(N+1)])
    for m in range(M):
        for i in range(m,M):
            rmi,lags = seismon.utils.xcorr(X[:,m]-np.mean(X[:,m]),X[:,i]-np.mean(X[:,i]),maxlags=N,normed=False)
            Rmi = scipy.linalg.toeplitz(np.flipud(rmi[range(N+1)]),r=rmi[range(N,2*N+1)])
            top = m*(N+1)
            bottom = (m+1)*(N+1)
            left = i*(N+1)
            right = (i+1)*(N+1)
            #R[range(top,bottom),range(left,right)] = Rmi

            for j in range(top,bottom):
                for k in range(left,right):
                    R[j,k] = Rmi[j-top,k-left]
         
            if not i == m:
                #R[range(left,right),range(top,bottom)] = Rmi  # Take advantage of hermiticity.

                RmiT = Rmi.T
                for j in range(left,right):
                    for k in range(top,bottom):
                        R[j,k] = RmiT[j-left,k-top]

    # Crosscorrelation vector.
    P = np.zeros([M*(N+1),])
    for i in range(M):
        top = i*(N+1)
        bottom = (i+1)*(N+1)

        p, lags = seismon.utils.xcorr(y-np.mean(y),X[:,i]-np.mean(X[:,i]),maxlags=N,normed=False)

        P[range(top,bottom)] = p[range(N,2*N+1)]

    # The following step is very inefficient because it fails to exploit the
    # block Toeplitz structure of R. Its done the same way in the builtin
    # function "firwiener".
    # P / R

    Z = np.linalg.lstsq(R.T, P.T)[0].T
    W = Z.reshape(M,N+1).T

    return W,R,P

def subtractFF(W,SS,S,samplef):

    # Subtracts the filtered SS from S using FIR filter coefficients W.
    # Routine written by Jan Harms. Routine modified by Michael Coughlin.
    # Modified: August 17, 2012
    # Contact: michael.coughlin@ligo.org

    N = len(W)-1
    ns = len(S)

    FF = np.zeros([ns-N,])

    for k in range(N,ns):
        tmp = SS[k-N:k+1,:] * W
        FF[k-N] = np.sum(tmp)

    cutoff = 1.0
    dataFF = gwpy.timeseries.TimeSeries(FF, sample_rate=samplef)
    #dataFFLowpass = dataFF.lowpass(cutoff, amplitude=0.9, order=3, method='scipy')
    #FF = np.array(dataFFLowpass)
    FF = np.array(dataFF)

    residual = S[range(ns-N)]-FF
    residual = residual - np.mean(residual)

    return residual, FF

def wiener_summary(params, target_channel, segment):
    """@calculates wiener filter for given channel and segment.

    @param params
        seismon params dictionary
    @param target_channel
        seismon channel structure
    @param segment
        [start,end] gps
    """

    gpsStart = segment[0]
    gpsEnd = segment[1]

    psdDirectory = params["dirPath"] + "/Text_Files/Wiener/" + target_channel.station_underscore + "/" + str(params["fftDuration"])

    directories = glob.glob(os.path.join(psdDirectory,"*"))

    data = {}

    for directory in directories:

        directorySplit = directory.split("/")
        N = int(directorySplit[-1])

        file = os.path.join(directory,"%d-%d.txt"%(gpsStart,gpsEnd))

        if not os.path.isfile(file):
            continue

        spectra_out = gwpy.frequencyseries.Spectrum.read(file)
        spectra_out.unit = 'counts/Hz^(1/2)'

        if np.sum(spectra_out.data) == 0.0:
            continue

        data[str(N)] = {}
        data[str(N)]["data"] = spectra_out

    if data == {}:
        return

    if params["doPlots"]:

        plotDirectory = params["path"] + "/wiener_summary/" + target_channel.station_underscore
        seismon.utils.mkdir(plotDirectory)

        fl, low, fh, high = seismon.NLNM.NLNM(2)

        pngFile = os.path.join(plotDirectory,"psd.png")
        lowBin = np.inf
        highBin = -np.inf
        plot = gwpy.plotter.Plot(figsize=[14,8])
        for key in data.iterkeys():

            label = key.replace("_","\_")

            plot.add_spectrum(data[key]["data"], label=label)
            lowBin = np.min([lowBin,np.min(data[key]["data"])])
            highBin = np.max([highBin,np.max(data[key]["data"])])

        kwargs = {"linestyle":"-.","color":"k"}
        plot.add_line(fl, low, **kwargs)
        plot.add_line(fh, high, **kwargs)
        plot.xlim = [params["fmin"],params["fmax"]]
        plot.ylim = [lowBin, highBin]
        plot.xlabel = "Frequency [Hz]"
        plot.ylabel = "Amplitude Spectrum [(m/s)/rtHz]"
        plot.add_legend(loc=1,prop={'size':10})
        plot.axes[0].set_xscale("log")
        plot.axes[0].set_yscale("log")

        plot.save(pngFile,dpi=200)
        plot.close()






"""
%prog

Michael Coughlin (coughlim@carleton.edu)

This program runs seismon_run.py.

"""

# =============================================================================
#
#                               PREAMBLE
#
# =============================================================================


import os, time
import numpy as np

__author__ = "Michael Coughlin <coughlim@carleton.edu>"
__date__ = "2012/2/7"
__version__ = "0.1"

gpsStart = 1009411215
numFrames = 24*366
#numFrames = 25*1
#numFrames = 2
frameDur = 3600
gpsEnd = 1009411215 + (numFrames+1)*frameDur

gpss = np.arange(gpsStart,gpsEnd,frameDur)

paramsFile = "/home/mcoughlin/Seismon/seismon/input/seismon_params_IRIS.txt"
framesFolder = "/home/mcoughlin/Stochastic/Seismic/frames/"
framesFolderCalibrated = "/home/mcoughlin/Stochastic/Seismic/frames_calibrated/"

for i in xrange(len(gpss)-1):

   if i < 8650:
      continue

   os.system("python seismon_run -p %s -s %d -e %d --doMakeFrames --doPlots --framesSampleRate 8 --framesFolder %s"%(paramsFile,gpss[i],gpss[i+1],framesFolder))

   os.system("python seismon_run -p %s -s %d -e %d --doMakeFramesCalibrated --doPlots --framesSampleRate 8 --framesFolder %s --framesFolderCalibrated %s"%(paramsFile,gpss[i],gpss[i+1],framesFolder,framesFolderCalibrated))




#Copyright 2015-2016 MathWorks, Inc.

# This template is used to generate an __init__.py file for a particular deployable package.

""" Package for executing deployed MATLAB functions """

from __future__ import print_function
import atexit
import glob
import importlib
import os
import os.path
import pdb
import platform
import sys
import weakref

class _PathInitializer(object):
    PLATFORM_DICT = {'Windows': ['PATH','dll',''], 'Linux': ['LD_LIBRARY_PATH','so','libmw'], 'Darwin': ['DYLD_LIBRARY_PATH','dylib','libmw']}
    SUPPORTED_PYTHON_VERSIONS = ['2_7', '3_3', '3_4', '3_5']
    RUNTIME_VERSION_W_DOTS = '9.1'
    RUNTIME_VERSION_W_UNDERSCORES = '9_1'
    PACKAGE_NAME = 'robustLocklossPredictionPkg'
    
    def set_interpreter_version(self):    
        """Make sure the interpreter version is supported."""
        ver = sys.version_info
        version = '{0}_{1}'.format(ver[0], ver[1])

        if version in _PathInitializer.SUPPORTED_PYTHON_VERSIONS:
            self.interpreter_version = version
        else:
            version_with_dot = version.replace("_", ".")
            raise EnvironmentError("Python {0} is not supported.".format(version_with_dot))

    def __init__(self):
        """Initialize the variables."""
        self.arch = ''
        self.is_linux = False    
        self.is_mac = False
        self.is_windows = False
        self.mr_handle = None
        self.ml_handle = None
        self.system = ''
        self.cppext_handle = None

        # path to the folder that stores Python extensions and shared libraries
        self.bin_dir = ''

        # path to the folder that stores pure Python matlab_pysdk.runtime code (_runtime_dir)
        self.runtime_dir = ''

        # path to the folder that stores the pure Python matlab mlarray code used for type conversion
        self.ml_dir = ''

        self.set_interpreter_version()
        self.get_platform_info()

        this_folder = os.path.dirname(os.path.realpath(__file__))
        self.path_file_name = os.path.join(this_folder, 'paths.{0}.txt'.format(self.arch))

        self.instances_of_this_package = set([])

            
    def read_path_file(self):
        """Look for a file that lists items to add to path. If present, read it and add the paths."""
        filtered_lines = []
        if os.path.isfile(self.path_file_name):
            pth_file = open(self.path_file_name, 'r')
            lines = pth_file.readlines()
            for line in lines:
                stripped_line = line.strip()
                if stripped_line and stripped_line[0] != '#':
                    filtered_lines.append(stripped_line)
        length = len(filtered_lines)
        if length == 3:
            (bin_dir, runtime_dir, ml_dir) = filtered_lines
            if (not os.path.isdir(bin_dir)) or (not os.path.isdir(runtime_dir)) or (not os.path.isdir(ml_dir)):
                return False
            else:
                (self.bin_dir, self.runtime_dir, self.ml_dir) = (bin_dir, runtime_dir, ml_dir)
                return True
        else:
            return False

    def write_path_file(self):
        """Write a file that lists items to add to path. If present, it will be overwritten."""
        existing_contents = ''
        if os.path.isfile(self.path_file_name):
            path_file = open(self.path_file_name, 'r')
            existing_contents = path_file.readlines()
            path_file.close()

        path_file = open(self.path_file_name, 'w')
        if self.system == 'Windows':
            print('# bin dir: added to both OS path and system path', file=path_file)
        else:
            print('# bin dir: added to system path', file=path_file)
        print(self.bin_dir, file=path_file)
        print('', file=path_file)
        
        print('# runtime dir: added to system path', file=path_file)
        print(self.runtime_dir, file=path_file)
        print('', file=path_file)

        print('# matlab (mlarray) dir: added to system path', file=path_file)
        print(self.ml_dir, file=path_file)
        print('', file=path_file)

        if existing_contents:
            print(existing_contents, file=path_file)
        path_file.close()
        
    def get_platform_info(self):
        """Ask Python for the platform and architecture."""
    
        # This will return 'Windows', 'Linux', or 'Darwin' (for Mac).
        self.system = platform.system() 
        if not self.system in _PathInitializer.PLATFORM_DICT:
            raise RuntimeError('{0} is not a supported platform.'.format(self.system))
        else:
            # path_var is the OS-dependent name of the path variable ('PATH', 'LD_LIBRARY_PATH', "DYLD_LIBRARY_PATH')
            (self.path_var, self.ext, self.lib_prefix) = _PathInitializer.PLATFORM_DICT[self.system]

        if self.system == 'Windows':
            self.is_windows = True
            bit_str = platform.architecture()[0]
            if bit_str == '64bit':
                self.arch = 'win64'
            elif bit_str == '32bit':
                self.arch = 'win32'
            else:
                raise RuntimeError('{0} is not supported.'.format(bit_str))
        elif self.system == 'Linux':
            self.is_linux = True
            self.arch = 'glnxa64'
        elif self.system == 'Darwin':
            self.is_mac = True
            self.arch = 'maci64'
        else:
            raise RuntimeError('Operating system {0} is not supported.'.format(self.system))
        
    def get_paths_from_os(self):
        """ 
        Look through the system path for a file whose name contains a runtime version
        corresponding to the one with which this package was produced.
        """

        # Concatenates the pieces into a string. The double parentheses are necessary.
        if self.system == 'Windows':
            file_to_find = ''.join((self.lib_prefix, 'mclmcrrt',
                 _PathInitializer.RUNTIME_VERSION_W_UNDERSCORES, '.', self.ext))
        elif self.system == 'Linux':
            file_to_find = ''.join((self.lib_prefix, 'mclmcrrt', '.', self.ext, '.',
                                    _PathInitializer.RUNTIME_VERSION_W_DOTS))
        elif self.system == 'Darwin':
            file_to_find = ''.join((self.lib_prefix, 'mclmcrrt', '.', 
                                    _PathInitializer.RUNTIME_VERSION_W_DOTS,
                                    '.', self.ext))
        else:
            raise RuntimeError('Operating system {0} is not supported.'.format(self.system))

        path_elements = []
        if self.path_var in os.environ:
            path_elements = os.environ[self.path_var].split(os.pathsep)
        if not path_elements:
            friendly_os_name = self.system
            if friendly_os_name == 'Darwin':
                friendly_os_name = 'Mac'
            raise RuntimeError('On {0}, you must set the environment variable "{1}" to a non-empty string. {2}'.format(
                friendly_os_name, self.path_var, 'For more details, see the package documentation.'))

        path_found = ''
        for elem in path_elements:
            filename = os.path.join(elem, file_to_find)
            if (os.path.isfile(filename)):
                path_found = elem
                break
        if not path_found:
            raise RuntimeError('Could not find an appropriate directory for MATLAB or the MATLAB runtime in {0}. Details: {1}'.format(
                self.path_var, file_to_find))

        # The last nonblank part of the path should be the arch (e.g., "win64").
        path_components = path_found.split(os.sep)
        if path_components[-1]:
            last_path_component = path_components[-1]
        else:
            # The directory name ended with a slash, so the last item in the list was an empty string. Go back one more.
            last_path_component = path_components[-2]

        if last_path_component != self.arch:
            raise RuntimeError('To call deployed MATLAB code on a {0} machine, you must run a {0} version of Python. Details: {1}'.format(
                last_path_component, path_found))
            
        matlabroot = os.path.dirname(os.path.dirname(os.path.normpath(path_found)))
        bin_dir = os.path.join(matlabroot, 'bin', self.arch)
        runtime_dir = os.path.join(matlabroot, 'toolbox', 'compiler_sdk', 'pysdk_py')
        ml_dir = os.path.join(runtime_dir, 'mlarray_dist')
        if not os.path.isdir(bin_dir):
            raise RuntimeError('Could not find the directory {0}'.format(bin_dir))
        if not os.path.isdir(runtime_dir):
            raise RuntimeError('Could not find the directory {0}'.format(runtime_dir))
        if not os.path.isdir(ml_dir):
            raise RuntimeError('Could not find the directory {0}'.format(ml_dir))
        (self.bin_dir, self.runtime_dir, self.ml_dir) = (bin_dir, runtime_dir, ml_dir)

    def update_paths(self):
        """Update the OS and Python paths."""

        #For Windows, add the bin_dir to the OS path. This is unnecessary
        #for Linux and Mac, where the OS can find this information via rpath.
        if self.is_windows:
            os.environ[self.path_var] = self.bin_dir + os.pathsep + os.environ[self.path_var]

        #Add all paths to the Python path.
        sys.path.insert(0, self.bin_dir)
        sys.path.insert(0, self.runtime_dir)
        sys.path.insert(0, self.ml_dir)

    def import_matlab_pysdk_runtime(self):
        """Import matlab_pysdk.runtime. Must be done after update_paths() and import_cppext() are called."""
        try:
            self.mr_handle = importlib.import_module('matlab_pysdk.runtime')
        except Exception as e:
            raise e

        if not hasattr(self.mr_handle, '_runtime_version_w_dots'):
            raise RuntimeError('Runtime version of package ({0}) does not match runtime version of previously loaded package'.format(
                _PathInitializer.RUNTIME_VERSION_W_DOTS))
        elif self.mr_handle._runtime_version_w_dots and (self.mr_handle._runtime_version_w_dots != _PathInitializer.RUNTIME_VERSION_W_DOTS):
            raise RuntimeError('Runtime version of package ({0}) does not match runtime version of previously loaded package ({1})'.format(
                _PathInitializer.RUNTIME_VERSION_W_DOTS,
                self.mr_handle._runtime_version_w_dots))
        else:
            self.mr_handle._runtime_version_w_dots = _PathInitializer.RUNTIME_VERSION_W_DOTS

        self.mr_handle._cppext_handle = self.cppext_handle

    def import_matlab(self):
        """Import the matlab package. Must be done after Python system path contains what it needs to."""
        try:
            self.ml_handle = importlib.import_module('matlab')
        except Exception as e:
            raise e

    def initialize_package(self):
        package_handle = self.mr_handle.DeployablePackage(self, self.PACKAGE_NAME, __file__)
        self.instances_of_this_package.add(weakref.ref(package_handle))
        package_handle.initialize()
        return package_handle

    def initialize_runtime(self, option_list):
        if not self.cppext_handle:
            raise RuntimeError('Cannot call initialize_application before import_cppext.')
        if self.is_mac:
            ignored_option_found = False
            for option in option_list:
                if option in ('-nodisplay', '-nojvm'):
                    ignored_option_found = True
                    break
            if ignored_option_found:
                print('WARNING: Options "-nodisplay" and "-nojvm" are ignored on Mac.')
                print('They must be passed to mwpython in order to take effect.')
        self.cppext_handle.initializeApplication(option_list)

    def terminate_runtime(self):
        if not self.cppext_handle:
            raise RuntimeError('Cannot call terminate_application before import_cppext.')
        self.cppext_handle.terminateApplication()

    def import_cppext(self):
        self.cppext_handle = importlib.import_module("matlabruntimeforpython" + self.interpreter_version)

try:
    _pir = _PathInitializer()
    _pir.get_paths_from_os()

    _pir.update_paths()
    _pir.import_cppext()
    _pir.import_matlab_pysdk_runtime()
    _pir.import_matlab()
except Exception as e:
    print("Exception caught during initialization of Python interface. Details: {0}".format(e))
    raise
    # We let the program exit normally.

def initialize():
    """ 
    Initialize package and return a handle.

    Initialize a package consisting of one or more deployed MATLAB functions. The return
    value is used as a handle on which any of the functions can be executed. To wait
    for all graphical figures to close before continuing, call wait_for_figures_to_close() 
    on the handle. To close the package, call terminate(), quit() or exit() (which are 
    synonymous) on the handle. The terminate() function is executed automatically when the 
    script or session ends.

    Returns
        handle - used to execute deployed MATLAB functions and to call terminate()
    """
    return _pir.initialize_package()

def initialize_runtime(option_list):
    """
    Initialize runtime with a list of startup options.

    Initialize the MATLAB Runtime with a list of startup options that will affect 
    all packages opened within the script or session. If it is not called 
    explicitly, it will be executed automatically, with an empty list of options,
    by the first call to initialize(). Do not call initialize_runtime() after 
    calling initialize().

    There is no corresponding terminate_runtime() call. The runtime is terminated
    automatically when the script or session ends.

    Parameters
        option_list - Python list of options; valid options are: 
                         -nodisplay (suppresses display functionality; Linux only)
                         -nojvm (disables the Java Virtual Machine)
    """
    if option_list:
        if not isinstance(option_list, list) and not isinstance(option_list, tuple):
            raise SyntaxError('initialize_runtime takes a list or tuple of strings.')
    _pir.initialize_runtime(option_list)

# terminate_runtime() is intentionally omitted. Instead, when running interactively, 
# the user should call exit(). When running a script, the runtime will automatically be
# terminated when the script ends.

@atexit.register
def __exit_packages():
    for package in _pir.instances_of_this_package:
        if package() is not None:
            package().terminate()




# Copyright (C) 2013 Michael Coughlin
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 3 of the License, or (at your
# option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

"""PDL client wrapper.

This script runs and monitors the PDL client.

Comments should be e-mailed to michael.coughlin@ligo.org.

"""

import os, sys, glob, optparse, shutil, warnings
import datetime, time
from subprocess import Popen, PIPE, STDOUT

__author__ = "Michael Coughlin <michael.coughlin@ligo.org>"
__version__ = 1.0
__date__    = "9/22/2013"

# =============================================================================
#
#                               DEFINITIONS
#
# =============================================================================

def verboseProcess(process,verbose):
   """@print PDL output (if requested)

   @param process
       pdl process
   @param verbose
       run verbosely (bool)
   """

   numLines = 10
   if verbose:
       # Poll process for new output until finished
       #while True:
       for i in xrange(numLines):
           nextline = process.stdout.readline()
           if nextline == '' and process.poll() != None:
               break
           sys.stdout.write(nextline)
           sys.stdout.flush()

def runProcess(process,verbose=False):
   """@run PDL process

   @param process
       pdl process
   @param verbose
       run verbosely (bool)
   """

   if not process == []:
       poll_process = process.poll()
       if not poll_process == None:
           process = []
       else:
           verboseProcess(process,verbose)
           return process


   runCommand = "rm /home/michael.coughlin/Seismon/ProductClient/data/receiver_index.db"
   process = Popen(runCommand,shell=True,stdin=PIPE, stdout=PIPE, stderr=STDOUT, close_fds=True)

   runCommand = "java -jar /home/michael.coughlin/Seismon/ProductClient/ProductClient.jar --receive --configFile=/home/michael.coughlin/Seismon/ProductClient/config.ini -Xmx1024m"
   process = Popen(runCommand,shell=True,stdin=PIPE, stdout=PIPE, stderr=STDOUT, close_fds=True)
   verboseProcess(process,verbose)
   exitCode = process.returncode

   return process

def findProcess():
    """@find PDL process
    """

    ps= Popen("ps -ef | grep michael.coughlin | grep ProductClient", shell=True, stdin=PIPE, stdout=PIPE, stderr=STDOUT, close_fds=True)
    output = ps.stdout.read()
    ps.stdout.close()
    ps.wait()

    lines = output.split("\n")

    process = []
    runningProcess = False
    for line in lines:
        if "source" in line:
            runningProcess = True
    if runningProcess:
        sys.exit()

    return process

folderDirectory = '/home/michael.coughlin/Seismon/ProductClient' 
dataDirectory = '/home/michael.coughlin/Seismon/ProductClient/data'
dataOldDirectory = '/home/michael.coughlin/Seismon/ProductClient/dataOld'

current_date = datetime.date.today()

process = findProcess()

while 1:

   today = datetime.date.today()
   yesterday = datetime.date.today() - datetime.timedelta(1)

   todayFolder = today.strftime("%Y_%m_%d")
   yesterdayFolder = yesterday.strftime("%Y_%m_%d")

   todayFolder = os.path.join(folderDirectory,todayFolder)
   yesterdayFolder = os.path.join(folderDirectory,yesterdayFolder)

   process = runProcess(process,verbose=True) 

   if not current_date == today:
       mv_command = "mv %s %s/%s"%(dataDirectory,dataOldDirectory,yesterdayFolder)
       os.system(mv_command)
       current_date = datetime.date.today()



import datetime
import json
import os
import io
import urllib.parse
import math
import re
import requests
import shutil
import tempfile
from itertools import chain

import numpy as np
from astropy.coordinates import SkyCoord
from astropy import time
import astropy.units as u
from astropy.table import Table
import pandas as pd
import matplotlib.style
import pkg_resources
from astropy.utils.data import get_pkg_data_filename
from astropy.io import fits

import matplotlib
matplotlib.use('Agg')
import matplotlib.pyplot as plt
from matplotlib.backends.backend_agg import FigureCanvasAgg as FigureCanvas
from matplotlib.figure import Figure
import cartopy.crs as ccrs

from flask import (
    abort, flash, jsonify, make_response, redirect, render_template, request,
    Response, url_for)
from flask_caching import Cache
from flask_login import (
    current_user, login_required, login_user, logout_user, LoginManager)
from wtforms import (
    BooleanField, FloatField, RadioField, TextField, IntegerField, StringField, PasswordField, SubmitField)
from wtforms_components.fields import (
    DateTimeField, DecimalSliderField, SelectField)
from wtforms import validators
from wtforms.validators import (
    DataRequired,
    Email,
    EqualTo,
    Length,
    Optional
)
from wtforms_alchemy.fields import PhoneNumberField
from passlib.apache import HtpasswdFile
from sqlalchemy.orm.exc import MultipleResultsFound, NoResultFound

from seismon.config import app
from seismon import models

#
#
# From http://wtforms-alchemy.readthedocs.io/en/latest/advanced.html#using-wtforms-alchemy-with-flask-wtf  # noqa: E501
from flask_wtf import FlaskForm
from wtforms_alchemy import model_form_factory
# The variable db here is a SQLAlchemy object instance from
# Flask-SQLAlchemy package


BaseModelForm = model_form_factory(FlaskForm)



class ModelForm(BaseModelForm):
    @classmethod
    def get_session(cls):
        return models.db.session
#
#
#

# Server-side cache for rendered view functions.
cache = Cache(app, config={
    'CACHE_DEFAULT_TIMEOUT': 86400,
    'CACHE_REDIS_HOST': 'redis',
    'CACHE_TYPE': 'redis'})

def one_or_404(query):
    # FIXME: https://github.com/mitsuhiko/flask-sqlalchemy/pull/527
    rv = query.one_or_none()
    if rv is None:
        abort(404)
    else:
        return rv


def human_time(*args, **kwargs):
    secs = float(datetime.timedelta(*args, **kwargs).total_seconds())
    units = [("day", 86400), ("hour", 3600), ("minute", 60), ("second", 1)]
    parts = []
    for unit, mul in units:
        if secs / mul >= 1 or mul == 1:
            if mul > 1:
                n = int(math.floor(secs / mul))
                secs -= n * mul
            else:
                n = secs if secs != int(secs) else int(secs)
            parts.append("%s %s%s" % (n, unit, "" if n == 1 else "s"))
    return parts[0]


@app.route('/')
def index():

    earthquakes = models.db.session.query(models.Earthquake).order_by(models.Earthquake.date.desc()).all()

    return render_template(
        'index.html',
        earthquakes=earthquakes)

@app.route('/earthquake/<event_id>/')
def earthquake(event_id):

    query = models.db.session.query(models.Earthquake).filter_by(event_id=event_id)

    try:
        earthquake = query.first()
    except NoResultFound:
        abort(404)
    predictions = models.db.session.query(models.Prediction).filter_by(event_id=event_id).all()

    return render_template(
            'earthquake.html',
            earthquake=earthquake,
            predictions=predictions)

@app.route('/earthquake/<event_id>/globemap.png')
def globemap(event_id):

    query = models.db.session.query(models.Earthquake).filter_by(event_id=event_id)

    try:
        earthquake = query.first()
    except NoResultFound:
        abort(404)

    ifos = models.db.session.query(models.Ifo).all()

    fig = Figure()
    ax = fig.add_subplot(1, 1, 1, projection=ccrs.Robinson())

    # make the map global rather than have it zoom in to
    # the extents of any plotted data
    ax.set_global()
    ax.stock_img()
    ax.coastlines()

    ax.plot(earthquake.lon, earthquake.lat, '*',
            markersize=15, color='k',
            transform=ccrs.PlateCarree())
    for ifo in ifos:
        ax.plot(ifo.lon, ifo.lat, 'o',
                markersize=6, color='r',
                transform=ccrs.PlateCarree())
        ax.text(ifo.lon+2, ifo.lat+2, ifo.ifo,
                transform=ccrs.PlateCarree())        

    plt.show()

    output = io.BytesIO()
    FigureCanvas(fig).print_png(output)
    return Response(output.getvalue(), mimetype='image/png')





"""Interact with external astrophysical alerts.

This script repeatedly queries the Gravitational Wave Candidate Event
Database (GraceDB) and populates the external trigger event channels
displayed on the MEDM screen in the control rooms.
"""

__author__ = 'Duncan Macleod <duncan.macleod@ligo.org>'

import os, sys, glob, optparse, warnings, time, json
import subprocess
import logging
import socket
import ssl
from argparse import ArgumentParser
from time import sleep
import numpy as np

import glue.datafind, glue.segments, glue.segmentsUtils, glue.lal

import epics
import cas

import seismon.utils

__author__ = "Michael Coughlin <michael.coughlin@ligo.org>"
__version__ = 1.0
__date__    = "9/22/2013"

# =============================================================================
#
#                               DEFINITIONS
#
# =============================================================================

# -----------------------------------------------------------------------------
# logging

BLACK, RED, GREEN, YELLOW, BLUE, MAGENTA, CYAN, WHITE = range(8)
RESET_SEQ = "\033[0m"
COLOR_SEQ = "\033[1;%dm"
BOLD_SEQ = "\033[1m"
COLORS = {
    'WARNING': YELLOW,
    'INFO': WHITE,
    'DEBUG': BLUE,
    'CRITICAL': RED,
    'ERROR': RED
}

class ColoredFormatter(logging.Formatter):
    """A `~logging.Formatter` that supports coloured output
    """
    def __init__(self, msg, use_color=True, **kwargs):
        logging.Formatter.__init__(self, msg, **kwargs)
        self.use_color = use_color

    def format(self, record):
        record.gpstime = gps_time_now(log=False)
        levelname = record.levelname
        if self.use_color and levelname in COLORS:
            levelname_color = (
                COLOR_SEQ % (30 + COLORS[levelname]) + levelname + RESET_SEQ)
            record.levelname = levelname_color
        return logging.Formatter.format(self, record)


class ExtAlertLogger(logging.Logger):
    """`~logging.Logger` with a nice format
    """
    FORMAT = ('[{bold}%(name)s{reset} %(gpstime)d] %(levelname)+19s: '
              '%(message)s'.format(bold=BOLD_SEQ, reset=RESET_SEQ))
    def __init__(self, name, level=logging.DEBUG):
        #super(Logger, self).__init__(name, level=level)
        #self.__init__(name, level=level) 
        logging.Logger.__init__(self,name,level=level)
        colorformatter = ColoredFormatter(self.FORMAT)
        console = logging.StreamHandler()
        console.setFormatter(colorformatter)
        self.addHandler(console)


# set logging
logger = ExtAlertLogger('ext-alert')


def unwrap_gracedb_event(event):
    """Parse a gracedb event into a `dict` with specific keys

    Parameters
    ----------
    event : `dict`
        raw event dict from GraceDb query

    Returns
    -------
    params : `dict`
        dict with the following keys:

        - 'ID': the event ID
        - 'TIME': the GPS time of the event,
        - 'SOURCE': the pipeline that created the event,
        - 'TYPE': the category of event,
        - 'PAUSE': the GPS time until which to pause tinj injections

    Raises
    ------
    NotImplementedError
        if the event doesn't match the 'G' or 'E' ID prefix, or
        if the event type hasn't been mapped to a pause time for tinj
    """
    uid = str(event['graceid'])
    out = {
        'ID': uid,
        'TIME': float(event['gpstime']),
        'SOURCE': str(event['pipeline']),
    }
    # GW candidate events (out-going)
    if uid.startswith('G'):
        out['TYPE'] = str(event['group'])
    # ExtTrig events (incoming)
    elif uid.startswith('E'):
        out['TYPE'] = str(event['search'])
    # Test events
    elif uid.startswith('T'):
        out['TYPE'] = 'TEST'
    else:
        raise NotImplementedError("Handling alerts for %s-type events has not "
                                  "been implemented" % uid[0])
    try:
        out['PAUSE'] = out['TIME'] + PAUSE[out['TYPE']]
    except KeyError:
        raise NotImplementedError("Unknown pause time for %r event "
                                  % out['TYPE'])
    return out

def gps_time_now(log=False):

    gps = int(os.popen("tconvert now").readline())
    return gps
    

def caput(channel, value, **kwargs):
    """caput with logging

    Parameters
    ----------
    channel : `str`
        name of channel to put
    value : `float`, `str`
        target value for channel
    **kwargs
        other keyword arguments to pass to `epics.caput`
    """
    epics.caput(channel, value, **kwargs)
    logger.debug("caput %s = %s" % (channel, value))


EPICS_RETRY = 0

def caget(channel, log=True, retry=0, **kwargs):
    """caget with logging

    Parameters
    ----------
    channel : `str`
        name of channel to get
    log : `bool`, optional, default: `True`
        use verbose logging
    retry : `int`, optional, defauilt: `0`
        catch failed cagets and try again this many times
    **kwargs
        other keyword arguments to pass to `epics.caget`

    Returns
    -------
    value : `float`, str`
        the value retrieved from that channel

    Raises
    ------
    ValueError
        if the channel failed to respond to a caget request
    """
    global EPICS_RETRY
    value = epics.caget(channel, **kwargs)
    if value is None:
        if log:
            logger.critical("Failed to caget %s" % channel)
        if EPICS_RETRY < retry:
            logger.warning('Retrying [%d]' % EPICS_RETRY)
            return caget(channel, log=log, retry=retry, **kwargs)
        else:
            raise ValueError("Failed to caget %s" % channel)
    if log:
        logger.debug("caget %s = %s" % (channel, value))
    EPICS_RETRY = 0
    return value

def parse_commandline():
    """@Parse the options given on the command-line.
    """
    parser = optparse.OptionParser(usage=__doc__,version=__version__)

    parser.add_option("-p", "--paramsFile", help="Seismon params file.",
                      default ="/Seismon/Seismon/seismon/input/seismon_params_earthquakesInfo.txt")

    parser.add_option("-s", "--gpsStart", help="GPS Start Time.", default=1126709046,type=int)
    parser.add_option("-e", "--gpsEnd", help="GPS End Time.", default=1126787937,type=int)

    parser.add_option("--eventfilesType", help="Event files type.", default="private")
    parser.add_option("--doPlots",  action="store_true", default=False)
    parser.add_option("--doEarthquakes",  action="store_true", default=False)
    parser.add_option("--doTimeseries",  action="store_true", default=False)
    parser.add_option("--doEPICs",  action="store_true", default=False)
    parser.add_option("--epicsChannelList", help="Seismon params file.",
                      default ="/Seismon/Seismon/seismon/input/seismon_epics_channel_list.txt")
    parser.add_option("--channelList", help="Seismon params file.",
                      default ="/Seismon/Seismon/seismon/input/seismon_nds_channel_list.txt")

    parser.add_option("--doReadEPICs",  action="store_true", default=False)

    parser.add_option("-v", "--verbose", action="store_true", default=False,
                      help="Run verbosely. (Default: False)")

    opts, args = parser.parse_args()

    # show parameters
    if opts.verbose:
        print >> sys.stderr, ""
        print >> sys.stderr, "running pylal_seismon_run..."
        print >> sys.stderr, "version: %s"%__version__
        print >> sys.stderr, ""
        print >> sys.stderr, "***************** PARAMETERS ********************"
        for o in opts.__dict__.items():
          print >> sys.stderr, o[0]+":"
          print >> sys.stderr, o[1]
        print >> sys.stderr, ""

    return opts

def params_struct(opts):
    """@Creates seismon params structure
    @param opts
        seismon command line options
    """

    params = seismon.utils.readParamsFromFile(opts.paramsFile)
    params["gpsStart"] = opts.gpsStart
    params["gpsEnd"] = opts.gpsEnd

    params["eventfilesType"] = opts.eventfilesType

    params["doPlots"] = opts.doPlots
    params["doEarthquakes"] = opts.doEarthquakes
    params["doTimeseries"] = opts.doTimeseries
    params["doEPICs"] = opts.doEPICs
    params["epicsChannelList"] = opts.epicsChannelList
    params["channelList"] = opts.channelList
    params["doReadEPICs"] = opts.doReadEPICs

    return params

def setup_params(params):

    gpsStart = params["gpsStart"]
    gpsEnd = params["gpsEnd"]

    params["channel"] = None
    params["referenceChannel"] = None
    params = seismon.utils.channel_struct(params,params["epicsChannelList"])
    #epicsDirectory = os.path.join(params["epicsDirectory"],"frames")
    #frameList = [os.path.join(root, name)
    #    for root, dirs, files in os.walk(epicsDirectory)
    #    for name in files]
    #datacache = []
    #for frame in frameList:
    #    thisFrame = frame.replace("file://localhost","")
    #    if thisFrame == "":
    #        continue
    
    #    thisFrameSplit = thisFrame.split(".")
    #    if thisFrameSplit[-1] == "log":
    #        continue
    
    #    thisFrameSplit = thisFrame.split("-")
    #    gps = float(thisFrameSplit[-2])
    #    dur = float(thisFrameSplit[-1].replace(".gwf",""))
    
    #    if gps+dur < gpsStart:
    #        continue
    #    if gps > gpsEnd:
    #        continue
    
    #    #cacheFile = glue.lal.CacheEntry("%s %s %d %d %s"%("XG","Homestake",gps,dur,frame))
    #    datacache.append(frame)
    #datacache = glue.lal.Cache(map(glue.lal.CacheEntry.from_T050017, datacache))
    #params["frame"] = datacache
    return params

# =============================================================================
#
#                                    MAIN
#
# =============================================================================

warnings.filterwarnings("ignore")

# Parse command line
opts = parse_commandline()
params = params_struct(opts)
params["path"] = params["dirPath"] + "/" + "all" + "/" + params["runName"] + "/" + "%.0f"%params["gpsStart"] + "-" + "%.0f"%params["gpsEnd"]
params["currentpath"] = params["dirPath"] + "/" + "all" + "/" + params["runName"] + "/" + "current"

logger.setLevel(opts.verbose * 10)

# -----------------------------------------------------------------------------
# execute action

socket.setdefaulttimeout(30)

CHANNELS = {}
ifos = ['H1','L1','V1','G1','MIT']
epicschannels = ['ETA','AMP','PROB','MULT']

for ifo in ifos:
    for epicschannel in epicschannels:
        channel = "%s:%s"%(ifo,epicschannel)
        CHANNELS[channel] = {'type': 'float'}

logger.info('Starting iteration')
# create PVdb
prefix = 'SEISMON:'
server = cas.CaServer(prefix, CHANNELS)
server.start()
logger.info("Created CA server with prefix %r" % server.prefix)

#try:
while True:
    while True:
        #segment = [gps_time_now()-60,gps_time_now()-30]
        #params["gpsStart"] = segment[0]
        #params["gpsEnd"] = segment[1]
        #params = setup_params(params)

        filename = os.path.join(params["epicsDirectory"],"epics.txt")
        lines = [line.rstrip('\n') for line in open(filename)]

        for line in lines:
            lineSplit = line.split(" ")
            ifo = lineSplit[0]
            prob = float(lineSplit[1])
            eta = float(lineSplit[2]) 
            amp = float(lineSplit[3])
            mult = float(lineSplit[4])

            name = "%s:ETA"%ifo
            caput("%s%s"%(prefix,name),eta)
            logger.info('Recorded %s' % name)
            name = "%s:PROB"%ifo
            caput("%s%s"%(prefix,name),prob)
            logger.info('Recorded %s' % name)
            name = "%s:AMP"%ifo
            caput("%s%s"%(prefix,name),amp)
            logger.info('Recorded %s' % name)
            name = "%s:MULT"%ifo
            caput("%s%s"%(prefix,name),mult)
            logger.info('Recorded %s' % name)

        logger.info('Iteration complete')
        time.sleep(1)
#except KeyboardInterrupt:
#    logger.warning('Caught keyboard interrupt, EXITING')




# Copyright (C) 2013 Michael Coughlin
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 3 of the License, or (at your
# option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

"""Seismic summary information generator.

This script generates a nested web page and content for
reviewing seismicity at the sites and earthquake status.

Comments should be e-mailed to michael.coughlin@ligo.org.

"""


import os, sys, glob, optparse, shutil, warnings

if not os.getenv("DISPLAY", None):
    import matplotlib
    matplotlib.use("agg", warn=False)

#import lal.gpstime
import seismon.eqmon

__author__ = "Michael Coughlin <michael.coughlin@ligo.org>"
__version__ = 1.0
__date__    = "9/22/2013"

# =============================================================================
#
#                               DEFINITIONS
#
# =============================================================================

def parse_commandline():
    """@Parse the options given on the command-line.
    """
    parser = optparse.OptionParser(usage=__doc__,version=__version__)

    parser.add_option("-p", "--paramsFile", help="Seismon params file.", 
                      default ="/Seismon/Seismon/seismon/input/seismon_params_earthquakesInfo.txt")

    parser.add_option("-s", "--gpsStart", help="GPS Start Time.", default=1126709046,type=float)
    parser.add_option("-e", "--gpsEnd", help="GPS End Time.", default=1126787937,type=float)
    parser.add_option("-i", "--ifos", help="Ifos.", default="H1,L1,G1,V1,MIT")

    parser.add_option("--eventfilesType", help="Event files type.", default="private")
    parser.add_option("--doPlots",  action="store_true", default=False)
    parser.add_option("--doEarthquakes",  action="store_true", default=False)
    parser.add_option("--doPurge",  action="store_true", default=False)
    parser.add_option("--doTimeseries",  action="store_true", default=False)
    parser.add_option("--doEPICs",  action="store_true", default=False)
    parser.add_option("--epicsChannelList", help="Seismon params file.",
                      default ="/Seismon/Seismon/seismon/input/seismon_epics_channel_list.txt")
    parser.add_option("--channelList", help="Seismon params file.",
                      default ="/Seismon/Seismon/seismon/input/seismon_nds_channel_list.txt")

    parser.add_option("--doReadEPICs",  action="store_true", default=False)

    parser.add_option("-v", "--verbose", action="store_true", default=False,
                      help="Run verbosely. (Default: False)")

    opts, args = parser.parse_args()

    # show parameters
    if opts.verbose:
        print >> sys.stderr, ""
        print >> sys.stderr, "running pylal_seismon_run..."
        print >> sys.stderr, "version: %s"%__version__
        print >> sys.stderr, ""
        print >> sys.stderr, "***************** PARAMETERS ********************"
        for o in opts.__dict__.items():
          print >> sys.stderr, o[0]+":"
          print >> sys.stderr, o[1]
        print >> sys.stderr, ""

    return opts

def params_struct(opts):
    """@Creates seismon params structure
    @param opts
        seismon command line options
    """

    params = seismon.utils.readParamsFromFile(opts.paramsFile)
    params["gpsStart"] = opts.gpsStart
    params["gpsEnd"] = opts.gpsEnd
    params["ifos"] = opts.ifos

    params["eventfilesType"] = opts.eventfilesType

    params["doPlots"] = opts.doPlots
    params["doEarthquakes"] = opts.doEarthquakes
    params["doPurge"] = opts.doPurge
    params["doTimeseries"] = opts.doTimeseries
    params["doEPICs"] = opts.doEPICs
    params["epicsChannelList"] = opts.epicsChannelList
    params["channelList"] = opts.channelList
    params["doReadEPICs"] = opts.doReadEPICs

    return params

# =============================================================================
#
#                                    MAIN
#
# =============================================================================

warnings.filterwarnings("ignore")

# Parse command line
opts = parse_commandline()
params = params_struct(opts)
params["path"] = params["dirPath"] + "/" + "all" + "/" + params["runName"] + "/" + "%.0f"%params["gpsStart"] + "-" + "%.0f"%params["gpsEnd"]
params["currentpath"] = params["dirPath"] + "/" + "all" + "/" + params["runName"] + "/" + "current"

if params["doEarthquakes"]:
    print(params["path"])
    segmentlist = seismon.eqmon.run_earthquakes_info(params,[params["gpsStart"],params["gpsEnd"]])

if params["doPurge"]:
    params["path"] = params["dirPath"] + "/" + "all" + "/" + params["runName"] + "/"
    #sys_command = "find %s/* -type d -mtime +1 -exec rm -rf {} \;"%params["path"]
    sys_command = "find %s* -type d -mmin +60 -exec rm -rf {} \;"%params["path"]
    os.system(sys_command)

